/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2003 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include <math.h>
#include <string.h>
#include "tms.h"

void act(struct small_creature *c)
    {
    long c_x, c_y;
    char message[256];
    struct clist *clcurr;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    correct_xy();
    c->nextaction = max(c->nextaction, game_variables->elapsed_units);

/*
 *
 * Check to see if the creature can see anything terrifying.  We have nested
 * if's because the can_see function is computationally expensive.
 *
 */

    if (!(flag_check(c->flags, INACTIVE)))
        {
        if (!(flag_check(c->flags, AFRAID)))
            {
            for(clcurr = current_level->creatures; (clcurr!=NULL); clcurr =
              clcurr->next)
                if (flag_check(clcurr->current->flags, TERRIFYING))
                    if (!(skill_check(c, c->def + 40 *
                      (c->identity == JANRA), DIFFICULT, 0) > 0))
/*
 *
 * Janra are variously described as mad, brave, and foolhardy.
 *
 */
                        if (can_see(c, clcurr->current->x, clcurr->current->y))
                                {
                                flag_set(&c->flags, AFRAID);
                                if (c == player)
                                    {
#ifdef HAS_SNPRINTF
                                    snprintf(message, 255,
                                      "You are paralyzed by terror at the sight of the ");
#else
                                    sprintf(message,
                                      "You are paralyzed by terror at the sight of the ");
#endif
                                    message[255] = '\0';
                                    strncat(message,
                                      master_creatures[clcurr->current->identity].species, max(0, 255 - strlen(message)));
                                    message[255] = '\0';
                                    strncat(message, "!", max(0, 255 - strlen(message)));
                                    message[255] = '\0';
                                    pmsg(message);
                                    }
                                }
            }
        else
            if (skill_check(c, c->def + 40 * (c->identity == JANRA),
              VERY_DIFFICULT, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE)
              > 0)
                {
                flag_reset(&c->flags, AFRAID);
                if (c == player)
                    pmsg("You regain control over yourself.");
                }
    /* move(((player->y - game_variables->display_center_y) + 6 + (LINES-10)/2),
    ((player->x - game_variables->display_center_x) + 13 + (COLS-16)/2)); */
        if (flag_check(c->flags, AFRAID))
            c->nextaction += current_level->timescale + SECOND;
        else if (flag_check(c->flags, SHIFT_MOVE) && (c->extended != NULL))
            {
            auto_move(c, c->extended->moving_direction_x, c->extended->
              moving_direction_y, SHIFT_MOVE);
            }
        else if (flag_check(c->flags, CONTROL_MOVE) && (c->extended != NULL))
            {
            auto_move(c, c->extended->moving_direction_x, c->extended->
              moving_direction_y, CONTROL_MOVE);
            }
        else
            {
            if (c==player)
                {
                get_command(c);
                if (flag_check(c->flags, HAS_LIGHT_SOURCE))
                    {
                    update_light_sources();
                    }
                if (flag_check(game_variables->flags, MAX_REFRESH))
                    update_window();
                }
            else
                {
                c_x = c->x;
                c_y = c->y;
                auto_act(c);
                if (flag_check(c->flags, HAS_LIGHT_SOURCE))
                    {
                    update_light_sources();
                    }
/*
 *
 * This code increases tremendously the runtime of the game, and sometimes
 * causes it to hang.  I am therefore having updates only before the player
 * moves, or if something significant happens.
 *
 */
                if (flag_check(game_variables->flags, MAX_REFRESH))
                    if  (((((game_variables->display_center_x - c_x <= COLS -
                      3) && (game_variables->display_center_y - c_y <= LINES -
                      1)) || ((game_variables->display_center_x - c->x <= COLS
                      - 3) && (game_variables->display_center_y - c->y <= LINES
                      - 1)))) && ((c_x != c->x) || (c_y != c->y)))
                        if ((can_see(player, c->x, c->y)) || (can_see(player,
                          c_x, c_y))) 
                            update_window();
                }
            }
        }
    else
        c->nextaction += current_level->timescale + SECOND;
    }

void allocate_experience(struct small_creature *c)
    {
    char message[256], player_input[256];
    long which, amount;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    /* c->skills->mod[0]=c->skills->mod[0]; */
    if (c->extended == NULL)
        pmsg("This creature cannot grow in experience.");
    else
        {
        which=skill_parse();
        if (which != -1)
            {
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "Allocate how much experience (%ld remaining)?",
              c->extended->unspent_ep);
#else
            sprintf(message, "Allocate how much experience (%ld remaining)?",
              c->extended->unspent_ep);
#endif
            message[255] = '\0';
            pmsg(message);
            tgets(player_input);
            sscanf(player_input, "%ld", &amount);
            if (amount > c->extended->unspent_ep)
                pmsg("You do not have that much experience remaining; aborted...");
            else if (amount <= 0)
                pmsg("Aborted...");
            else
                if (allocate_experience_internal(c, which, amount) > 0)
                    {
                    nocap_pmsg("(Not all of the stated experience was allocated; the unallocated experience was");
                    nocap_pmsg("not sufficient to reach the next level beyond the level reached by the");
                    nocap_pmsg("experience which was allocated.)");
                    }
            }
        }
    calc_ready(c);
    }

void fs_allocate_experience(struct small_creature *c)
    {
    char message[256], player_input[256];
    long which, amount;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    /* c->skills->mod[0]=c->skills->mod[0]; */
    if (c->extended == NULL)
        pmsg("This creature cannot grow in experience.");
    else
        {
        which=fs_skill_parse();
        if (which != -1)
            {
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "Allocate how much experience (%ld remaining)?",
              c->extended->unspent_ep);
#else
            sprintf(message, "Allocate how much experience (%ld remaining)?",
              c->extended->unspent_ep);
#endif
            message[255] = '\0';
            pmsg(message);
            tgets(player_input);
            sscanf(player_input, "%ld", &amount);
            if (amount > c->extended->unspent_ep)
                pmsg("You do not have that much experience remaining; aborted...");
            else if (amount <= 0)
                pmsg("Aborted...");
            else
                if (allocate_experience_internal(c, which, amount) > 0)
                    {
                    nocap_pmsg("(Not all of the stated experience was allocated; the unallocated experience was");
                    nocap_pmsg("not sufficient to reach the next level beyond the level reached by the");
                    nocap_pmsg("experience which was allocated.)");
                    }
            }
        }
    calc_ready(c);
    }

/* Returns the amount left over, if the amount used is not enough to reach the
next increment.  */

long allocate_experience_internal(struct small_creature *c, long which, long
  amount)
    {
    long cost, remaining, my_continue;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    remaining = 0;
    if (c->extended == NULL)
        pmsg("This creature cannot grow in experience.");
    else
        {
        my_continue = 1;
        remaining = amount;
        while (my_continue)
            {
            cost=my_exp(c->extended->skills->bv[which]+1)-
              my_exp(c->extended->skills->bv[which]);
            if (cost>remaining)
                my_continue = 0;
            else
                {
                ++c->extended->skills->bv[which];
                ++c->extended->skills->av[which];
                c->extended->skills->ep[which] += cost;
                c->extended->unspent_ep-=cost;
                c->extended->total_ep+=cost;
                c->extended->effective_ep+=cost;
                remaining -= cost;
                gain_related_skills(c,which); /* recursive */
                }
            }
        }
    calc_ready(c);
    return remaining;
    }

void gain_related_skills(struct small_creature *c, long which)
    {
    long i;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    for(i=0;i<NUM_LEARNING_DIFFERENCES;++i)
        {
        if (learning_differences[i][0]== which)
            check_skill_gain(c, which,
                c->extended->skills->bv[which] - learning_differences[i][2]);
        if (learning_differences[i][1]== which)
            check_skill_gain(c, which,
                c->extended->skills->bv[which] - learning_differences[i][2]);
        }
    }

void check_skill_gain(struct small_creature *c, long sn, long amount)
    {
    long old;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    old=c->extended->skills->ub[sn];
    if (old < amount)
        {
        c->extended->skills->ub[sn]=amount;
        if (my_exp(amount)>my_exp(old))
            {
            c->extended->skills->bv[sn]=my_log(amount +
              c->extended->skills->ep[sn]);
            c->extended->skills->av[sn]=c->extended->skills->bv[sn]+
              c->extended->skills->mod[sn];
            gain_related_skills(c, sn);
            }
        }
    }

void auto_act(struct small_creature *c)
    {
    long acted;
    struct small_creature *target;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    acted=FALSE;
    if (!(c->hostile==NULL))
        {
        target = nearest_hostile(c);
        if (target != NULL)
            {
            /* 
            if (clear_path(c->x, c->y, target->x, target->y))
                {
                attack_msl(c, target);
                acted = TRUE;
                }
            else */ if (attempt_move(c, target->x, target->y))
                acted = TRUE;
            }
        }
    else if (!(c->follow == NULL))
        {
        if (distance(c->x, c->y, c->follow->current->x, c->follow->current->y)
          > 2)
            if (attempt_move(c, c->follow->current->x, c->follow->current->y))
                acted = TRUE;
        }
    if (!(acted))
        {
        if (flag_check(c->flags, C_WANDERING))
            move_creature(c, trandom() % 3 - 1, trandom() % 3 - 1);
        take_time(c, SECOND);
        }
    }

void get_command(struct small_creature *c)
    {
    char command; 
    struct small_item *this_item;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    /* char message[256];
    sprintf(message, "Time in units: %ld", game_variables->elapsed_units);
    pmsg(message); */
    if (find_light_source_radius(c) > 0)
        update_light_sources();
    update_window();
    command=tgetc();
    game_variables->messages_printed=0;
    switch(command)
        {
        case LEFT: case SECOND_LEFT: move_creature(c, -1, 0); break;
        case DOWN: case SECOND_DOWN: move_creature(c, 0, 1); break;
        case UP: case SECOND_UP: move_creature(c, 0, -1); break;
        case RIGHT: case SECOND_RIGHT: move_creature(c, 1, 0); break;
        case UP_LEFT: case SECOND_UP_LEFT: move_creature(c, -1, -1); break;
        case UP_RIGHT: case SECOND_UP_RIGHT: move_creature(c, 1, -1); break;
        case DOWN_LEFT: case SECOND_DOWN_LEFT: move_creature(c, -1, 1); break;
        case DOWN_RIGHT: case SECOND_DOWN_RIGHT: move_creature(c, 1, 1); break;
        case JOG_LEFT: case JOG_DOWN: case JOG_UP: case JOG_RIGHT: case
          JOG_UP_LEFT: case JOG_UP_RIGHT: case JOG_DOWN_LEFT: case
          JOG_DOWN_RIGHT: case RUN_LEFT: case RUN_DOWN: case RUN_UP: case
          RUN_RIGHT: case RUN_UP_LEFT: case RUN_UP_RIGHT: case RUN_DOWN_LEFT:
          case RUN_DOWN_RIGHT:
            if (c->extended != NULL)
                {
                switch(command)
                    {
                    case JOG_LEFT: case RUN_LEFT: case JOG_UP_LEFT: case
                      RUN_UP_LEFT: case JOG_DOWN_LEFT: case RUN_DOWN_LEFT:
                        c->extended->moving_direction_x = -1;
                    break;
                    case JOG_UP: case RUN_UP: case JOG_DOWN: case RUN_DOWN:
                        c->extended->moving_direction_x = 0;
                    break;
                    case JOG_RIGHT: case RUN_RIGHT: case JOG_UP_RIGHT: case
                      RUN_UP_RIGHT: case JOG_DOWN_RIGHT: case RUN_DOWN_RIGHT:
                        c->extended->moving_direction_x = 1;
                    break;
                    }
                switch(command)
                    {
                    case JOG_UP_LEFT: case RUN_UP_LEFT: case JOG_UP: case
                      RUN_UP: case JOG_UP_RIGHT: case RUN_UP_RIGHT:
                        c->extended->moving_direction_y = -1;
                    break;
                    case JOG_LEFT: case RUN_LEFT: case JOG_RIGHT: case
                      RUN_RIGHT:
                        c->extended->moving_direction_y = 0;
                    break;
                    case JOG_DOWN_LEFT: case RUN_DOWN_LEFT: case JOG_DOWN: case
                      RUN_DOWN: case JOG_DOWN_RIGHT: case RUN_DOWN_RIGHT:
                        c->extended->moving_direction_y = 1;
                    break;
                    }
                switch(command)
                    {
                    case JOG_LEFT: case JOG_DOWN: case JOG_UP: case
                      JOG_RIGHT: case JOG_UP_LEFT: case JOG_UP_RIGHT: case
                      JOG_DOWN_LEFT: case JOG_DOWN_RIGHT:
                        if (current_level != countryside)
                            flag_set(&c->flags, CONTROL_MOVE);
                        move_creature(c, c->extended->moving_direction_x,
                          c->extended->moving_direction_y);
                    break;
                    case RUN_LEFT: case RUN_DOWN: case RUN_UP: case RUN_RIGHT:
                      case RUN_UP_LEFT: case RUN_UP_RIGHT: case RUN_DOWN_LEFT:
                      case RUN_DOWN_RIGHT:
                        if (current_level != countryside)
                            flag_set(&c->flags, SHIFT_MOVE);
                        move_creature(c, c->extended->moving_direction_x,
                          c->extended->moving_direction_y);
                    break;
                    }
                }
            else
                pmsg("This creature cannot move in that manner.");
        break;
        case ABRIDGED_HELP:
            nocap_pmsg("y k u    7 8 0  <-- Motion keys.  Important keystroke commands are: ");
            nocap_pmsg(" \\|/      \\|/   ");
            nocap_pmsg("h-@-l or 4-@-6  ? This help              D followed by 'charity contacts', and");
            nocap_pmsg(" /|\\      /|\\   / Complete command list  D followed by 'extended help' are");
            game_variables->messages_printed = 3;
            nocap_pmsg("b j n    1 2 3  X Examine something      both of substantial interest.");
        break;
        case ACTIVATE: activate(c, get_item(c, EXTRACT_RECURSIVE)); break;
        case ACTIVATE_SPACE: space_effect(c, c->x, c->y); break;
        case ALLOCATE_EXPERIENCE: allocate_experience(c); break;
        case CITY_MOVE: city_move(c); break;
        case CLOSE_DOOR: close_door(c); break;
        case CLOSE_ITEM: close_item(get_item(c, EXTRACT_RECURSIVE)); break;
        case DO_ACTION: do_action(c); break;
        case DROP: drop_item(c); break;
        case ENTER_OR_EXIT: change_level(player->x, player->y); break;
        case EQUIP: wear_wield(c); break;
        case EXAMINE_ITEM:
            describe_item(get_item(c, EXTRACT_RECURSIVE));
        break;
        case EXAMINE_SPACE: examine(c); break;
        case FIRE_THROW: attack_msl(c, loc_creature(c)); break;
        case FLY: action_fly(c); break;
        case GIVE: give_item(c, loc_creature(c)); break;
        case GO_TO_SLEEP:
            /* Modify later to check for hostile creatures first.  */
            go_to_sleep(c);
            if (c->extended != NULL)
                pass_time(c->extended->stati->time[STATUS_SLEEPING]);
        break;
        case HELP: read_file(FILE_HELP); break;
        case INSPECT_SEARCH: inspect_search(c); break;
        case INVENTORY: print_inventory(c); break;
        case MUNCH:
            this_item = get_item(c, EXTRACT_RECURSIVE);
            if (this_item != NULL)
                {
                if (orig_items[this_item->identity].effect == E_FOOD)
                    use_item(c, this_item);
                else
                    pmsg("That isn't food.");
                }
        break;
        case OPEN_DOOR: open_door(c); break;
        case OPEN_ITEM: open_item(get_item(c, EXTRACT_RECURSIVE)); break;
        case PRAY: pray(c, 0); break;
        case PRINT_COPYRIGHT: read_file(FILE_COPYRIGHT); break;
        case QUAFF:
            this_item = get_item(c, EXTRACT_RECURSIVE);
            if (this_item != NULL)
                {
                if ((orig_items[this_item->identity].effect == ALCOHOL) ||
                  (orig_items[this_item->identity].effect == QUENCH_THIRST))
                    use_item(c, this_item);
                else
                    pmsg("That is not something one would ordinarily drink.");
                }
        break;
        case QUIT: if (confirm_neg()) my_finish(0); break;
        case READ_ITEM: read_item(c, get_item(c, EXTRACT_RECURSIVE)); break;
        case REFRESH: refresh_screen(); break;
        case REST_AWHILE: rest(c); break;
        case REST_ONCE:
                if (!(c->extended==NULL))
                    c->nextaction += max(1,
                      UNITS_PER_SECOND*current_level->scale/c->extended->move);
                else
                    c->nextaction += max(1, UNITS_PER_SECOND*
                      current_level->scale/master_creatures[c->identity].move);
        break;
        case ROLE_PLAY_INFO: role_play_info(); break;
        case SAVE: save( save_filename ); break;
        case TALK: talk(c, one_move_loc_creature(c)); break;
        case THANKS: read_file(FILE_THANKS); break;
        case UNEQUIP: unequip(c); break;
        case USE_SKILL: use_skill(c); break;
        case VAULT: jump(c); break;
        case WALK: flag_reset(&c->flags, RUNNING); break;
        case WRITE_ON_ITEM: find_and_write_on_item(c); break;
        default: pmsg("I don't know how to do that.  Hit '?' for help.");
        break;
        }
    }

long move_creature(struct small_creature *c, long plus_x, long plus_y) 
/* or attempt to, at any rate */
/* later attempt to update to deal with flying */
    {
    long success, i, j;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    success = 0;
    if ((!(flag_check(c->flags, IMMOBILE))) && (!(status_check(c,
      STATUS_IMMOBILE) > 0)))
        {
        if ((current_level == countryside) && (status_check(c, STATUS_LOST) >
          0))
            {
            plus_x = (trandom() % 3) - 1;
            plus_y = (trandom() % 3) - 1;
            while ((plus_x == 0) && (plus_y == 0))
                {
                plus_x = (trandom() % 3) - 1;
                plus_y = (trandom() % 3) - 1;
                }
            if (c == player)
                pmsg("You wander around, trying to find your way...");
            }
        if ((0 <= c->x+plus_x) && (MAX_X-1 >= c->x+plus_x) &&
        (0 <= c->y+plus_y) && (MAX_Y-1 >= c->y+plus_y))
            {
            if (flag_check(current_level->space[c->x+plus_x][c->y+plus_y].
              flags, CAN_OCCUPY) &&
              (current_level->space[c->x + plus_x][c->y + plus_y].creatures ==
              NULL))
                {
                if (flag_check(current_level->space[c->x+plus_x][c->y+plus_y].
                  flags,
                  EFFECT_BEFORE_MOVE))
                    {
                    if (space_effect(c, c->x+plus_x, c->y+plus_y))
                        success = 1;
                    }
                else
                    success = 1;
                if (flag_check(master_creatures[c->identity].flags, WATER_ONLY)
                  && !(flag_check(current_level->space[c->x + plus_x][c->y +
                  plus_y].flags, IS_WATER)))
                    success = 0;
                if (success)
                    {
                    if (c->extended != NULL)
                        fatigue(c, min(c->extended->penalty, 10) *
                          min(current_level->scale, MINUTE) * (1 + 9 *
                          (flag_check(c->flags, RUNNING))));
                    flag_reset(&current_level->space[c->x][c->y].flags,
                      OCCUPIED);
                    remove_creature_from_clist(c,
                      &current_level->space[c->x][c->y].creatures);
                    c->x+=plus_x;
                    c->y+=plus_y;
                    for(i = max(0, c->x - 1); i < min(c->x + 2, MAX_X); ++i)
                        for(j = max(0, c->y - 1); j < min(c->y + 2, MAX_Y);
                          ++j)
                            if (flag_check(current_level->space[i][j].flags,
                              HAS_SECRET))
                                if ((c->extended != NULL) && ((c == player) ||
                                  (is_following(c, player))))
                                    if (skill_check(c,
                                      c->extended->skills->av[SEARCH],
                                      VERY_DIFFICULT,
                                      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE)
                                      > 0)
                                        {
                                        discover_secret(i, j);
                                        }
                    append_creature_to_clist(c,
                      &current_level->space[c->x][c->y].creatures);
                    flag_set(&current_level->space[c->x][c->y].flags,
                      OCCUPIED);
                    if (!(c->extended==NULL))
                        c->nextaction += max(1, UNITS_PER_SECOND*
                          current_level->scale/c->extended->move) *
                          (1 + 4 * (!(flag_check(c->flags, RUNNING))) * (1 + 2
                          * (flag_check(c->flags, HIDDEN))));
                    else
                        c->nextaction += max(1, UNITS_PER_SECOND*
                          current_level->scale/master_creatures[c->identity].
                            move)
                          * (1 + 4 * (!(flag_check(c->flags, RUNNING))) * (1 +
                          2 * (flag_check(c->flags, HIDDEN))));
                    if ((flag_check(current_level->space[c->x][c->y].flags,
                      AUTOMATIC_EFFECT)) && (!(flag_check(current_level->space
                      [c->x][c->y].flags, EFFECT_BEFORE_MOVE))))
                        space_effect(c, c->x, c->y);
                    if ((c == player) && (current_level == countryside))
                        {
                        game_variables->countryside_player_x = c->x;
                        game_variables->countryside_player_y = c->y;
                        }
                    pickup(c);
                    }
                }
            else
                {
                if (current_level->space[c->x + plus_x][c->y +
                  plus_y].creatures != NULL)
                    attack_hth(c, current_level->space[c->x+plus_x]
                      [c->y+plus_y].creatures->current);
                else
                    if ((c==player) && ((plus_x != 0) || (plus_y != 0)))
                        pmsg("You can't move there.");
                }
            }
        else
            if (c==player)
                pmsg("You can't move there.");
        }
    else
        {
        if (c == player)
        {
            pmsg("You can't move.");
        pass_time(MINUTE);
        }
        success = 0;
        }
    return success;
    }

void refresh_screen(void)
    {
    clear();
    update_window();
    refresh();
    }

void activate(struct small_creature *c, struct small_item *i)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    use_item(c, i);
    }

void close_item(struct small_item *i)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) i);
#endif
    if (i != NULL)
        {
        if (flag_check(i->flags, CAN_OPEN))
            {
            if (flag_check(i->flags, IS_OPEN))
                {
                flag_reset(&i->flags, IS_OPEN);
                pmsg("Closed.");
                }
            else
                pmsg("It is already closed!");
            }
        else
            pmsg("It is not clear how one would go about closing such a thing.");
        }
    }

void attack_common(struct small_creature *attacker, struct small_creature
  *attackee, long flags)
    {
    long result, attack;
    char message[256], attacker_title[256], attackee_title[256];
    struct clist *traverse;
    struct small_item *weapon;
    if (current_level == countryside)
        {
        attacker->nextaction += current_level->timescale;
        return;
        }
#ifdef DEBUG
    check_pointer((struct pointer_struct *) attacker);
    check_pointer((struct pointer_struct *) attackee);
#endif
    attack = 0;
    find_title(attacker, attacker_title);
    find_title(attackee, attackee_title);
    if ((attacker==player) && (flag_check(attackee->flags, HUMAN)))
        {
        if (confirm_neg())
            attack=1;
        }
    else if ((flag_check(attacker->flags, HUMAN) && flag_check(attackee->flags,
      HUMAN)) || (attacker->identity == attackee->identity))
        {
        attack=0;
        reset_hostility(attacker, attackee);
        }
    else if ((flag_check(current_level->space[attacker->x][attacker->y].flags,
      IS_SANCTUARY)) ||
      (flag_check(current_level->space[attackee->x][attackee->y].flags,
      IS_SANCTUARY)))
        {
        attack = 0;
        if (attackee == player)
            pmsg("The aegis of God shields you!");
        }
    else
        attack=1;
    if (!(is_hostile(attacker, attackee)))
        attack &= ((flag_check(attacker->flags, BELLICOSE)) && 
          (!(is_following(attackee, attacker))) && (!(is_following(attacker,
          attackee))));
    if (flag_check(flags, ATTACK_IS_HTH))
        if ((flag_check(attacker->flags, MISSLE_ATTACK)) &&
          (flag_check(attacker->flags, HUMAN)))
            {
            attack = 0;
            if (attacker == player)
                pmsg("You are wielding a missle weapon.");
            }
    if (flag_check(flags, ATTACK_IS_MSL))
        if (!(flag_check(attacker->flags, MISSLE_ATTACK)))
            {
            attack = 0;
            if (attacker == player)
                pmsg("You are not wielding a missle weapon.");
            }
    if (attack)
        {
    status_reset(attacker, STATUS_MIRACLE_AEGIS_OF_GOD);
        take_time(attacker, attacker->attack_time);
        fatigue(attacker, attacker->attack_time);
        if (flag_check(attackee->flags, HUMAN))
            flag_set(&attacker->flags, HAS_SINNED);
        make_hostile(attackee, attacker);
        if (!(attackee->followed_by == NULL))
            for(traverse = attackee->followed_by; (traverse != NULL);
              traverse = traverse->next)
        if (traverse->current != NULL)
                if (!(flag_check(traverse->current->flags, NO_DEFEND_MASTER)))
                    make_hostile(traverse->current, attacker);
        result=skill_check(attacker, attacker->att, attackee->def,
          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        if (result>0)
            {
            if (flag_check(flags, ATTACK_IS_HTH))
                {
                weapon = find_weapon(attacker);
                if (weapon != NULL)
                    {
                    if (weapon->uses > 0)
                        --weapon->uses;
                    }
                }
            if (can_see(player, attacker->x, attacker->y))
                {
                switch(result)
                    {
                    case 1:
                        attack_str(message, attacker, attackee, flags|
                          ATTACK_HIT);
                    break;
                    case 2:
                        attack_str(message, attacker, attackee, flags|
                          ATTACK_HIT|ATTACK_ONE_CRITICAL_SUCCESS);
                    break;
                    case 3:
                        attack_str(message, attacker, attackee, flags|
                          ATTACK_HIT|ATTACK_TWO_CRITICAL_SUCCESSES);
                    break;
                    default:
                        attack_str(message, attacker, attackee, flags|
                          ATTACK_HIT|ATTACK_THREE_CRITICAL_SUCCESSES);
                    break;
                    }
                pmsg(message);
                }
            else if (attacker == player)
                pmsg("You hit it.");
            else if (attackee == player)
                pmsg("It hits you.");
            if (flag_check(attacker->flags, C_BLUNT_DAMAGE))
                {
                if (creature_damage(attackee, attacker->dmg + (10 * result -
                  10), CREATURE_DAMAGE_BLUNT_DAMAGE|CREATURE_DAMAGE_WAKE_UP|
                  CREATURE_DAMAGE_ARMOR_ABSORBS, result))
                    {
                    gain_experience(attacker, attackee->epvalue);
                    creature_death(attackee);
                    }
                }
            else
                {
                if (creature_damage(attackee, attacker->dmg + (10 * result -
                  10), CREATURE_DAMAGE_WAKE_UP|CREATURE_DAMAGE_ARMOR_ABSORBS,
                  result))
                    {
                    gain_experience(attacker, attackee->epvalue);
                    creature_death(attackee);
                    }
                }
            }
        else if (result<0)
            {
            if (flag_check(flags, ATTACK_IS_HTH))
                {
                weapon = find_weapon(attacker);
                if (weapon != NULL)
                    {
                    if (weapon->uses > 0)
                        --weapon->uses;
                    }
                }
            if (attacker==player)
                pmsg("You blunder and hit yourself.");
            else if (can_see(player, attacker->x, attacker->y))
                {
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s blunders.", attacker_title);
#else
                sprintf(message, "%s blunders.", attacker_title);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            if (flag_check(attacker->flags, C_BLUNT_DAMAGE))
                {
                if (creature_damage(attacker, attacker->dmg * result,
                  CREATURE_DAMAGE_WAKE_UP|CREATURE_DAMAGE_BLUNT_DAMAGE|
                  CREATURE_DAMAGE_ARMOR_ABSORBS, result))
                    creature_death(attacker);
                }
            else
                {
                if (creature_damage(attacker, attacker->dmg * result,
                  CREATURE_DAMAGE_WAKE_UP|CREATURE_DAMAGE_ARMOR_ABSORBS,
                  result))
                    creature_death(attacker);
                }
            }
        else if (can_see(player, attacker->x, attacker->y))
            {
            if (attacker == player)
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "You miss %s.", attackee_title);
#else
                sprintf(message, "You miss %s.", attackee_title);
#endif
            else
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s misses %s.", attacker_title,
                  attackee_title);
#else
                sprintf(message, "%s misses %s.", attacker_title,
                  attackee_title);
#endif
            message[255] = '\0';
            pmsg(message);
            }
        else if (attacker == player)
            pmsg("You miss it.");
        else if (attackee == player)
            pmsg("It misses you.");
        /* Take account for the time taken. */
        take_time(attacker, attacker->attack_time);
        }
    else /* If the attacker won't attack, he will swap places to get past.  */
        if ((flag_check(flags, ATTACK_IS_HTH)) &&
          (!(flag_check(current_level->space[attackee->x][attackee->y].flags,
          IS_SANCTUARY))))
            swap_places(attacker, attackee);
    }

void attack_hth(struct small_creature *attacker, struct small_creature
  *attackee)
    {
    char attackee_title[256], message[256];
    long attack;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) attacker);
    check_pointer((struct pointer_struct *) attackee);
#endif
    attack = TRUE;
    if (flag_check(attackee->flags, ABOVE_GROUND))
        {
        if (!(flag_check(attacker->flags, ABOVE_GROUND)))
            {
            attack = FALSE;
            if (attacker == player)
                {
                find_title(attackee, attackee_title);
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s is flying over your head!",
                  attackee_title);
#else
                sprintf(message, "%s is flying over your head!",
                  attackee_title);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            }
        }
    else
        {
        if (flag_check(attacker->flags, ABOVE_GROUND))
            land_from_flying(attacker);
        }
    if (attack)
        {
        flag_reset(&attacker->flags, HIDDEN);
        attack_common(attacker, attackee, ATTACK_IS_HTH);
        if (flag_check(attackee->flags, HITS_BACK))
            attack_common(attackee, attacker, ATTACK_IS_HTH);
        }
    else
        take_time(attacker, 1 + SECOND/10);
    }

void attack_msl(struct small_creature *attacker, struct small_creature
  *attackee)
    {
    struct small_item *weapon;
    long success, item_out_of_charges;
    item_out_of_charges = 0;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) attacker);
    check_pointer((struct pointer_struct *) attackee);
#endif
    success = 0;
    if (attackee != NULL)
        {
        if (clear_path(attacker->x, attacker->y, attackee->x, attackee->y))
            success = TRUE;
        if (flag_check(attacker->flags, HUMAN))
            {
            weapon = find_weapon(attacker);
            if (weapon != NULL)
                {
                success = (use_item(attacker, weapon));
                if (!success)
                    item_out_of_charges = 1;
                }
            else
                pmsg("You need to wield a weapon.");
            }
        if (success)
            {
            if (flag_check(master_creatures[attacker->identity].
              missle_attack_messages, BREATHES_FIRE_ON))
                projectile_path(attacker->x, attacker->y, attackee->x,
                  attackee->y, '*');
            else
                projectile_path(attacker->x, attacker->y, attackee->x,
                  attackee->y, ')');
            attack_common(attacker, attackee, ATTACK_IS_MSL);
            }
        else if (attacker == player)
            {
            if (!item_out_of_charges)
                pmsg("The path is obstructed.");
            }
        }
    else
        {
        if (attacker == player)
            pmsg("That is not a valid creature.");
        }
    }

void open_item(struct small_item *i)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) i);
#endif
    if (i != NULL)
        {
        if ((flag_check(i->flags, CAN_OPEN)) && ((flag_check(i->flags,
          CONTAINER)) && ((flag_check(i->flags, IDENTIFIED)) ||
          (!(flag_check(i->flags, MAY_HAVE_COMPARTMENT))))))
            {
            if (!(flag_check(i->flags, IS_OPEN)))
                {
                if (!(flag_check(i->flags, LOCKED)))
                    {
                    pmsg("You open it.");
                    flag_set(&i->flags, IS_OPEN);
                    }
                else
                pmsg("Hmm...  It appears to be locked.");
                }
            else
                pmsg("It is already open.");
            }
        else
            pmsg("It is not clear how one would go about opening such a thing.");
        }
    }

void give_item(struct small_creature *c, struct small_creature *ctarget)
    {
    struct small_item *i;
    char creature_title[256], message[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) ctarget);
#endif
    if (ctarget != NULL)
        {
        i = get_item(c, EXTRACT_RECURSIVE|EXTRACT_REMOVE_ITEM);
        if (i != NULL)
            {
            find_title(ctarget, creature_title);
            if (ctarget->extended != NULL)
                {
                add_item(&ctarget->extended->pack, i);
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s accepts your gift.", creature_title);
#else
                sprintf(message, "%s accepts your gift.", creature_title);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            else
                {
                add_item(&current_level->space[c->x][c->y].items, i);
#ifdef HAS_SNPRINTF
                snprintf(message, 255, "%s doesn't care for your offering, and drops it on the ground.", creature_title);
#else
                sprintf(message, "%s doesn't care for your offering, and drops it on the ground.", creature_title);
#endif
                message[255] = '\0';
                pmsg(message);
                }
            }
        }
    }

void inspect_search(struct small_creature *c)
    {
    long i, j;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    pmsg("You search the area for any secrets...");
    take_time(c, min(HOUR, MINUTE + current_level->scale * UNITS_PER_SECOND));
    if (c->extended != NULL)
        for(i = max(0, c->x - 1); i < min(MAX_X, c->x + 2); ++i)
            for(j = max(0, c->y - 1); j < min(MAX_Y, c->y + 2); ++j)
                if (flag_check(current_level->space[i][j].flags, HAS_SECRET))
                    if (skill_check(c, c->extended->skills->av[SEARCH],
                      DIFFICULT, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE)
                      > 0)
                        discover_secret(i, j);
    }

void print_attributes(struct small_creature *c)
    {
    int i;
    char message[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    clear();
    fs_pmsg("Explain abbreviations?");
    if (confirm_silent())
        read_file(FILE_PRINT_ATTRIBUTES_ABBREVIATIONS);
    else
        clear();
#ifdef HAS_SNPRINTF
    snprintf(message, 255, "Ag: %ld, AL: %ld, Ch: %ld, Co: %ld, De: %ld, In: %ld",
      c->extended->attributes[AG], c->extended->attributes[AL],
      c->extended->attributes[CH], c->extended->attributes[CO],
      c->extended->attributes[DE], c->extended->attributes[IN]);
#else
    sprintf(message, "Ag: %ld, AL: %ld, Ch: %ld, Co: %ld, De: %ld, In: %ld",
      c->extended->attributes[AG], c->extended->attributes[AL],
      c->extended->attributes[CH], c->extended->attributes[CO],
      c->extended->attributes[DE], c->extended->attributes[IN]);
#endif
    message[255] = '\0';
    fs_pmsg(message);
#ifdef HAS_SNPRINTF
    snprintf(message, 255, "Kn: %ld, Me: %ld, Pe: %ld, Sp: %ld, St: %ld, Wi: %ld",
      c->extended->attributes[KN], c->extended->attributes[ME],
      c->extended->attributes[PE], c->extended->attributes[SPEED],
      c->extended->attributes[ST], c->extended->attributes[WI]);
#else
    sprintf(message, "Kn: %ld, Me: %ld, Pe: %ld, Sp: %ld, St: %ld, Wi: %ld",
      c->extended->attributes[KN], c->extended->attributes[ME],
      c->extended->attributes[PE], c->extended->attributes[SPEED],
      c->extended->attributes[ST], c->extended->attributes[WI]);
#endif
    message[255] = '\0';
    fs_pmsg(message);
#ifdef HAS_SNPRINTF
    snprintf(message, 255, "Att: %ld, Dmg: %ld, Def: %ld, Hv: %ld/%ld",
      c->extended->att, c->dmg, c->def, c->hv, c->extended->phv);
#else
    sprintf(message, "Att: %ld, Dmg: %ld, Def: %ld, Hv: %ld/%ld",
      c->extended->att, c->dmg, c->def, c->hv, c->extended->phv);
#endif
    message[255] = '\0';
    fs_pmsg(message);
    fs_pmsg("");
    for(i=0;i<NUM_SKILLS;++i)
        {
#ifdef HAS_SNPRINTF
        snprintf(message, 255, "%s: ub: %ld, bv: %ld, ep: %ld, gaa: %ld, av: %ld",
          skill_info[i].name, c->extended->skills->ub[i],
          c->extended->skills->bv[i], c->extended->skills->ep[i],
          c->extended->skills->mod[i], c->extended->skills->av[i]);
#else
        sprintf(message, "%s: ub: %ld, bv: %ld, ep: %ld, gaa: %ld, av: %ld",
          skill_info[i].name, c->extended->skills->ub[i],
          c->extended->skills->bv[i], c->extended->skills->ep[i],
          c->extended->skills->mod[i], c->extended->skills->av[i]);
#endif
        message[255] = '\0';
        fs_pmsg(message);
        }
    fs_anykey();
    clear();
    update_window();
    }

void read_item(struct small_creature *c, struct small_item *i)
    {
    /*
    char message[256];
    */
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    if (i != NULL)
        {
        if (flag_check(i->flags, CAN_READ))
            use_item(c, i);
        else
            {
            pmsg("You can't read that.");
            /*
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "%ld", i->flags);
#else
            sprintf(message, "%ld", i->flags);
#endif
            message[255] = '\0';
            pmsg(message);
            */
            }
        }
    }

void creature_death(struct small_creature *c)
    {
    char message[256], title[256], corpse_name[256];
    struct small_item *corpse;
    struct clist *traverse;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c==player)
        player_death();
    switch(c->identity)
        {
        case ROC:
            flag_set(&game_variables->quests_completed, SLEW_ROC);
        break;
        case KRAKEN:
            flag_set(&game_variables->quests_completed, SLEW_KRAKEN);
        break;
        case BEHEMOTH:
            flag_set(&game_variables->quests_completed, SLEW_BEHEMOTH);
        break;
        case GIANT_SCORPION:
            ++game_variables->scorpion_stings_found;
            if (game_variables->scorpion_stings_found >= 12)
                flag_set(&game_variables->quests_completed,
                  DOZEN_GIANT_SCORPIONS);
        break;
        case LEVIATHAN:
            flag_set(&game_variables->quests_completed, SLEW_LEVIATHAN);
        break;
        case TERRASK:
            flag_set(&game_variables->quests_completed, SLEW_TERRASK);
        break;
        }
    if (!(c->name == NULL))
        strncpy(title, c->name->text, 255);
    else if (!(c->extended == NULL))
        strncpy(title, c->extended->title, 255);
    else
#ifdef HAS_SNPRINTF
        snprintf(title, 255, "The %s", master_creatures[c->identity].species);
#else
        sprintf(title, "The %s", master_creatures[c->identity].species);
#endif
    title[255] = '\0';
#ifdef HAS_SNPRINTF
    snprintf(message, 255, "%s dies.", title);
#else
    sprintf(message, "%s dies.", title);
#endif
    message[255] = '\0';
    if (c->extended != NULL)
        {
        append_ilist(&current_level->space[c->x][c->y].items,
          c->extended->pack);
        c->extended->pack = NULL;
        append_ilist(&current_level->space[c->x][c->y].items,
          c->extended->wear_wield);
        c->extended->wear_wield = NULL;
        }
    if (can_see(player, c->x, c->y))
        pmsg(message);
    if (!(c->extended==NULL))
        if (!(c->extended->mount==NULL))
            dismount(c);
    if (!(is_on_clist(c, current_level->space[c->x][c->y].creatures)))
        correct_xy();
    remove_creature_from_clist(c,
      &current_level->space[c->x][c->y].creatures);
    flag_reset(&current_level->space[c->x][c->y].flags, OCCUPIED);
    remove_creature_from_clist(c, &current_level->creatures);
    for(traverse=current_level->creatures;(traverse!=NULL);traverse=
      traverse->next)
        if (traverse->current != NULL)
        {
        reset_follow(c, traverse->current);
        reset_follow(traverse->current, c);
        reset_hostility(traverse->current, c);
        if (!(traverse->current->extended == NULL))
            if (traverse->current->extended->mount==c)
                dismount(traverse->current);
        }
    corpse=generate_item(current_level, MORTAL_REMAINS);
    small_item_make_normal(corpse);
    if (!(c->name == NULL))
#ifdef HAS_SNPRINTF
        snprintf(corpse_name, 255, "body of %s", c->name->text);
#else
        sprintf(corpse_name, "body of %s", c->name->text);
#endif
    else if (!(c->extended == NULL))
        strncpy(corpse_name, c->extended->body, 255);
    else
        strncpy(corpse_name, master_creatures[c->identity].body, 255);
    corpse_name[255] = '\0';
    corpse->other = c->identity;
    name_item(corpse, corpse_name);
    if (!(c->extended == NULL))
        corpse->weight = c->extended->natural_weight;
    else
        corpse->weight = master_creatures[c->identity].natural_weight;
    corpse->uses=corpse->weight/(2*LB);
    if (!(c->extended == NULL))
        corpse->value = c->extended->value / 10;
    else
        corpse->value = master_creatures[c->identity].value / 10;
    append_creature_to_clist(c, &corpse->creatures);
    place_item(current_level, corpse, c->x, c->y);
    update_window();
    }

void player_death(void)
    {
    struct small_creature *new_player;
    pmsg("You died!");
    morewait();
    new_player = appoint_successor(player);
    if (new_player != NULL)
        {
        player = new_player;
        pmsg("You pass on your mantle to a successor...");
        morewait();
        pmsg("The game will now take the perspective of your successor as player.");
        morewait();
        }
    else
    {
        read_file(FILE_TOMBSTONE);
        my_finish(0);
    }
    }

long use_item(struct small_creature *c, struct small_item *i)
    {
    long temp, result, x, y;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
    check_pointer((struct pointer_struct *) i);
#endif
    if (i != NULL)
        {
        if ((orig_items[i->identity].maxuses > 0) && (i->uses <= 0))
            {
            pmsg("Pfft.  All out.");
            return FALSE;
            }
        else
            {
            result = TRUE;
            switch (orig_items[i->identity].effect)
                {
                case NONE:
                    pmsg("Nothing happens.");
                break;
                case E_ADRENALINE:
                    pmsg("You inject it...");
                    status_add(c, STATUS_ADRENALINE, 1);
                    calc_ready(c);
                    take_time(c, 5L*SECOND);
                break;
                case AID_OPEN:
                    aid_open(c);
                break;
                case ALCOHOL:
                    pmsg("You drink it...");
                    status_add(c, STATUS_ALCOHOL, 1);
                    status_add(c, STATUS_THIRSTY, -1);
                    calc_ready(c);
                    take_time(c, MINUTE);
                break;
                case BREATHE_UNDERWATER:
                    status_add(c, STATUS_BREATHE_WATER, 1);
                break;
                case E_CALCULATOR:
                    calculator();
                break;
                case CATCH_FISH:
                    go_fishing(c);
                break;
                case E_ANTIDOTE:
                    antidote(c, loc_creature(c));
                break;
                case E_CLOCKWORK_DEVICE:
                    /* They default to calculator.  */
                    calculator();
                break;
                case E_COFFEE:
                    pmsg("You drink it...");
                    status_add(c, STATUS_COFFEE, 1);
                    status_add(c, STATUS_THIRSTY, -1);
                    calc_ready(c);
                    take_time(c, MINUTE);
                case E_EQUIP:
                    pmsg("Use the command 'e', equip, to use this item.");
                break;
                case FIND_DIRECTION:
                    status_reset(c, STATUS_LOST);
                break;
                case FIREBREATHER:
                    if (c->identity == TUZ)
                        pmsg("You toss it back.");
                    else
                        {
                        pmsg("*cough!* *sputter!* Whew!");
                        pmsg("That's, um, one liquor with a bit of a kick!");
                        pmsg("Your mouth is on fire!");
                        pmsg("You need to put out the fire!");
                        status_add(c, STATUS_THIRSTY, 2);
                        }
                    status_add(c, STATUS_ALCOHOL, 1);
                    take_time(c, MINUTE);
                break;
                case E_FLY:
                    /* Check that this is properly used.  */
                    flag_set(&c->flags, FLYING);
                break;
                case E_FOOD:
                    switch(trandom() % 10)
                        {
                        case 0:
                            pmsg("Yum!");
                        break;
                        case 1:
                            pmsg("Hmm...  a pleasant surprise.");
                        break;
                        case 2:
                            pmsg("Delicious!");
                        break;
                        case 3:
                            pmsg("Hmm...  needs spices.");
                        break;
                        case 4:
                            pmsg("Perfect!");
                        break;
                        case 5:
                            pmsg("Benissimo!");
                        break;
                        case 6:
                            pmsg("That tasted wonderful!");
                        break;
                        case 7:
                            pmsg("Mmmmmm...");
                        break;
                        case 8:
                            pmsg("Excellent.");
                        break;
                        case 9:
                            pmsg("Not bad...");
                        break;
                        }
                    status_add(c, STATUS_HUNGRY, -1);
                    calc_ready(c);
                    take_time(c, 15L*MINUTE);
                break;
                case GIVES_LIGHT:
                    /* Implement later.  */
                break;
                case E_HELSRIANA_TEA:
                    pmsg("You drink it...");
                    status_add(c, STATUS_HELSRIANA, 1);
                    status_add(c, STATUS_THIRSTY, -1);
                    calc_ready(c);
                    take_time(c, MINUTE);
                break;
                case E_HORN:
                    e_horn(c);
                break;
                case E_HUMMER:
                    status_add(c, STATUS_HUMMER, 1);
                break;
                case IGNITE:
                    pmsg("Ignite:");
                    ignite(c, get_item(c, EXTRACT_RECURSIVE|
                      EXTRACT_PROMPT_BOTH_PACK_AND_EQUIPMENT));
                break;
                case E_INCENSE:
                    status_add(c, STATUS_PLEASING, 1);
                break;
                case E_JANRA_BALL:
                    e_janra_ball(c);
                break;
                case E_KEY:
                    pmsg("Which door?");
                    if (loc_xy(c, &x, &y))
                        {
                        if ((current_level->space[x][y].type == L_DOOR) &&
                          (flag_check(current_level->space[x][y].flags,
                          DOOR_LOCKED)))
                            {
                            pmsg("The key slides in, and the lock clicks open.");
                            flag_reset(&current_level->space[x][y].flags,
                              DOOR_LOCKED);
                            if (c->extended != NULL)
                                find_and_remove_id(&c->extended->pack, KEY);
                            }
                        else
                            pmsg("That is not a door with a lock you can use this key on.");
                        }
                break;
                case E_MANUAL_OF_SKILL:
                    if (c->extended != NULL)
                        {
                        if (skill_check(c,
                          c->extended->skills->av[READING_WRITING],
                          VERY_DIFFICULT,
                          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE) > 0)
                            {
                            pmsg("You become more skillful!");
                            if (flag_check(i->flags, FINE_QUALITY))
                                {
                                c->extended->unspent_ep += 10;
                                temp = allocate_experience_internal(c,
                                  i->other, 10);
                                }
                            else if (flag_check(i->flags, POOR_QUALITY))
                                {
                                c->extended->unspent_ep += 2;
                                temp = allocate_experience_internal(c,
                                  i->other, 2);
                                }
                            else
                                {
                                c->extended->unspent_ep += 5;
                                temp = allocate_experience_internal(c,
                                  i->other, 5);
                                }
                            if (temp > 0)
                                {
                                c->extended->unspent_ep -= temp;
                                increment_skill_one_unit(c, i->other);
                                }
                            pass_days(MONTH);
                            }
                        else
                            {
                            pmsg("You puzzle over it for a day, but can't make it out.");
                            pass_days(1);
                            }
                        }
                break;
                case E_MEDICINE:
                    medicine(c, loc_creature(c));
                break;
                case E_MIRROR:
                    switch(trandom()%2)
                        {
                        case 0:
                            pmsg("You see your reflection.");
                        break;
                        case 1:
                            pmsg("What an ugly picture!");
                        break;
                        }
                break;
                case E_MUSIC_BOX:
                    pmsg("Twinkle, twinkle, little star...");
                break;
                case E_MUSIC_FLUTE:
                    music(c, MUSIC_FLUTE);
                break;
                case E_MUSIC_HARP:
                    music(c, MUSIC_HARP);
                break;
                case E_MUSIC_LUTE:
                    music(c, MUSIC_LUTE);
                break;
                case E_MUSIC_ORGAN:
                    music(c, MUSIC_ORGAN);
                break;
                case ONE_ARROW:
                    if (c->extended != NULL)
                        {
                        result = find_and_remove_id(&c->extended->wear_wield,
                          LONGBOW_ARROW);
                        if (result == FALSE)
                            pmsg("You need to wield arrows to use this weapon.");
                        }
                    else
                        result = TRUE;
                break;
                case ONE_BOLT:
                    if (c->extended != NULL)
                        {
                        result = find_and_remove_id(&c->extended->wear_wield,
                          CROSSBOW_BOLT);
                        if (result == FALSE)
                            pmsg("You need to wield crossbow bolts to use this weapon.");
                        }
                    else
                        result = TRUE;
                break;
                case E_PERFUME:
                    pmsg("You spray a little on yourself...");
                    status_add(c, STATUS_PLEASING, 1);
                    calc_ready(c);
                break;
                case E_POISON:
                    /* Implement later.  */
                break;
                case E_SHOOT_DART:
                    if (i->uses > 0)
                        {
                        // i->uses--;
                        result = TRUE;
                        }
                    else
                        {
                        // pmsg("Out of darts.");
                        result = FALSE;
                        }
                break;
                case E_SHOOT_ROCKET:
                    if (i->uses > 0)
                        {
                        // i->uses--;
                        result = TRUE;
                        }
                    else
                        {
                        // pmsg("Out of rockets.");
                        result = FALSE;
                        }
                break;
                case QUENCH_THIRST:
                    pmsg("You drink it...");
                    status_add(c, STATUS_THIRSTY, -1);
                    calc_ready(c);
                    take_time(c, MINUTE);
                break;
                case E_TALINOR_TEA:
                    pmsg("You drink it...");
                    status_add(c, STATUS_TALINOR, 1);
                    status_add(c, STATUS_THIRSTY, -1);
                    calc_ready(c);
                    take_time(c, MINUTE);
                break;
                case TELLS_TIME:
                    pmsg("You read from it the present time.");
                break;
                case E_TEXT:
                    if (can_see(c, c->x, c->y))
                        {
                        if (i->writing != NULL)
                            {
                            pmsg("You begin to read...");
                            morewait();
                            if (c != NULL)
                                if (c->extended != NULL)
                                    {
                                    if (skill_check(c,
                                      c->extended->skills->av[READING_WRITING],
                                      DIFFICULT,
                                      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE)
                                      > 0)
                                        {
                                        if (flag_check(i->flags2,
                                          WRITING_IS_FILENAME))
                                            read_file(i->writing->text);
                                        else
                                            read_text_struct(i->writing);
                                        }
                                    else
                                        pmsg("You can't make out the writing.");
                                    }
                            }
                        else
                            pmsg("It is blank.");
                        }
                    else
                        pmsg("You can't see the text well enough to read it.");
                break;
                case WARM_ENVIRONMENT:
                    status_add(c, STATUS_COLD, -1);
                break;
                }
            if (orig_items[i->identity].maxuses > 0)
                {
                --i->uses;
                if (i->number > 1)
                    --i->number;
                if (flag_check(i->flags, DISAPPEAR_LAST_USE))
                    {
                    i->weight -= i->weight/(i->uses + 1);
                    i->value -= i->value/(i->uses + 1);
                    if (((i->uses <= 0) || (i->number <= 0)) && (c->extended !=
                      NULL))
                        find_and_remove_item(i, &c->extended->pack);
                    calc_ready(c);
                    }
                }
            return result;
            }
        }
    return TRUE;
    }
