/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2003 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include <sys/types.h>
#include <curses.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "tms.h"

char tgetc(void)
    {
    game_variables->messages_printed = 0;
    game_variables->fs_messages_printed = 0;
    /* The next line is intended as a possible means of addressing a freezing */
    /* bug.  */
    fflush(stdin);
    return getch();
    }

void ensure_synced_screen(void)
        {
#ifdef SPECIAL_WNOUTREFRESH
        wrefresh(top_text);
#endif
        }

long fs_pmsg(char *message)
    {
    long i;
    char local_message[256];
#ifdef DEBUG
    check_string(message);
#endif
    strncpy(local_message, message, 255);
    local_message[255] = '\0';
    /* Strip a trailing newline */
    for(i=0;i<256;++i)
        if ((local_message[i] == 10) || (local_message[i] == 13) ||
          (local_message[i] == 0))
            {
            local_message[i] = 0;
            i = 256;
            }
    ++game_variables->fs_messages_printed;
    mvprintw(min(LINES - 1, game_variables->fs_messages_printed - 1), 0,
      "%s\n", local_message);
    clrtoeol();
    i = TRUE;
    if ((game_variables->fs_messages_printed >= LINES - 2) &&
      (flag_check(game_variables->flags, MESSAGES_PROMPT)))
        {
        game_variables->fs_messages_printed=0;
        i=fs_morewait(); /* have morewait refresh */
        clear();
        }
    refresh();
    return i;
    }

long rf_fs_pmsg(char *message)
    {
    long i;
    char local_message[256];
#ifdef DEBUG
    check_string(message);
#endif
    strncpy(local_message, message, 255);
    local_message[255] = '\0';
    /* Strip a trailing newline */
    for(i=0;i<256;++i)
        if ((local_message[i] == 10) || (local_message[i] == 13) ||
          (local_message[i] == 0))
            {
            local_message[i] = 0;
            i = 256;
            }
    ++game_variables->fs_messages_printed;
    mvprintw(min(LINES - 1, game_variables->fs_messages_printed - 1), 0,
      "%s\n", local_message);
    clrtoeol();
    i = READ_FILE_CONTINUE;
    if ((game_variables->fs_messages_printed >= LINES - 2) &&
      (flag_check(game_variables->flags, MESSAGES_PROMPT)))
        {
        game_variables->fs_messages_printed=0;
        i=rf_fs_morewait(); /* have morewait refresh */
        clear();
        }
    refresh();
    return i;
    }

long cap_fs_pmsg(char *message)
    {
    long i;
    char local_message[256];
#ifdef DEBUG
    check_string(message);
#endif
    strncpy(local_message, message, 255);
    local_message[255] = '\0';
    /* Capitalize first letter. */
    if ((local_message[0] >= 'a') && (local_message[0] <= 'z'))
        local_message[0] += 'A' - 'a';
    /* Strip a trailing newline */
    for(i=0;i<256;++i)
        if ((local_message[i] == 10) || (local_message[i] == 13) ||
          (local_message[i] == 0))
            {
            local_message[i] = 0;
            i = 256;
            }
    ++game_variables->fs_messages_printed;
    if ((game_variables->fs_messages_printed >= LINES - 1) &&
      (flag_check(game_variables->flags, MESSAGES_PROMPT)))
        {
        game_variables->fs_messages_printed=0;
        i=fs_morewait(); /* have morewait refresh */
        clear();
        }
    else
        i = TRUE;
    mvprintw(min(LINES - 1, game_variables->fs_messages_printed - 1), 0,
      "%s\n", local_message);
    clrtoeol();
    refresh();
    return i;
    }

long pmsg(char *message)
    {
    char local_message[256];
    long i;
#ifdef DEBUG
    check_string(message);
#endif
    strncpy(local_message, message, 255);
    local_message[255] = '\0';
    /* Capitalize first letter. */
    if ((local_message[0] >= 'a') && (local_message[0] <= 'z'))
        local_message[0] += 'A' - 'a';
    ++game_variables->messages_printed;
    if ((game_variables->messages_printed >= 5) &&
      (flag_check(game_variables->flags, MESSAGES_PROMPT)))
        {
        game_variables->messages_printed=0;
        i = morewait();
        }
    else
        i = TRUE;
    mvwprintw(top_text, 5, 0, "%s\n", local_message);
    wclrtoeol(top_text);
#ifdef NO_WNOUTREFRESH
    wrefresh(top_text);
#else
    wnoutrefresh(top_text);
#endif
        ensure_synced_screen();
    return i;
    }

long noprompt_pmsg(char *message)
    {
    char local_message[256];
#ifdef DEBUG
    check_string(message);
#endif
    strncpy(local_message, message, 255);
    local_message[255] = '\0';
    /* Capitalize first letter. */
    if ((local_message[0] >= 'a') && (local_message[0] <= 'z'))
        local_message[0] += 'A' - 'a';
    ++game_variables->messages_printed;
    if ((game_variables->messages_printed >= 5) &&
      (flag_check(game_variables->flags, MESSAGES_PROMPT)))
        {
        game_variables->messages_printed=0;
        }
    mvwprintw(top_text, 5, 0, "%s\n", local_message);
    wclrtoeol(top_text);
#ifdef NO_WNOUTREFRESH
    wrefresh(top_text);
#else
    wnoutrefresh(top_text);
#endif
    return 1; /* Presently not configured to return nonzero character.  */
    }

long nocap_pmsg(char *message)
    {
    long i;
#ifdef DEBUG
    check_string(message);
#endif
    fflush(stdout);
    ++game_variables->messages_printed;
    if ((game_variables->messages_printed >= 5) && (flag_check(game_variables->flags, MESSAGES_PROMPT)))
        {
        game_variables->messages_printed=0;
        i = morewait();
        }
    else
        i = TRUE;
    mvwprintw(top_text, 5, 0, "%s\n", message);
    wclrtoeol(top_text);
#ifdef NO_WNOUTREFRESH
    wrefresh(top_text);
#else
    wnoutrefresh(top_text);
#endif
    return i;
    }

void left_panel_update(void)
    {
    mvwprintw(left_panel, 0, 0, "Ag: %ld/%ld",
      player->extended->attributes[AG], player->extended->perm_attributes[AG]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 1, 0, "AL: %ld/%ld",
      player->extended->attributes[AL], player->extended->perm_attributes[AL]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 2, 0, "Ch: %ld/%ld",
      player->extended->attributes[CH], player->extended->perm_attributes[CH]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 3, 0, "Co: %ld/%ld",
      player->extended->attributes[CO], player->extended->perm_attributes[CO]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 4, 0, "De: %ld/%ld",
      player->extended->attributes[DE], player->extended->perm_attributes[DE]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 5, 0, "In: %ld/%ld",
      player->extended->attributes[IN], player->extended->perm_attributes[IN]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 6, 0, "Kn: %ld/%ld",
      player->extended->attributes[KN], player->extended->perm_attributes[KN]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 7, 0, "Me: %ld/%ld",
      player->extended->attributes[ME], player->extended->perm_attributes[ME]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 8, 0, "Pe: %ld/%ld",
      player->extended->attributes[PE], player->extended->perm_attributes[PE]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 9, 0, "Sp: %ld/%ld",
      player->extended->attributes[SPEED], player->extended->perm_attributes[SPEED]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 10, 0, "St: %ld/%ld",
      player->extended->attributes[ST], player->extended->perm_attributes[ST]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 11, 0, "Wi: %ld/%ld",
      player->extended->attributes[WI], player->extended->perm_attributes[WI]);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 13, 0, "Att: %ld", player->att);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 14, 0, "Dmg: %ld", player->dmg);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 15, 0, "Def: %ld", player->def);
    wclrtoeol(left_panel);
    /*
    mvwprintw(left_panel, 16, 0, "HV: %ld/%ld",
      player->extended->phv,
      2 * player->extended->phv -
      player->extended->phv-player->extended->stati->time[STATUS_WOUNDED]);
    */
    mvwprintw(left_panel, 16, 0, "HV: %ld/%ld",
      max_player_phv - player->extended->stati->time[STATUS_WOUNDED],
      max_player_phv);
    wclrtoeol(left_panel);
    mvwprintw(left_panel, 17, 0, "Wgt: %ld", player->extended->curr_weight/LB);
    wclrtoeol(left_panel);
#ifdef NO_WNOUTREFRESH
    wrefresh(left_panel);
#else
    wnoutrefresh(left_panel);
#endif
    }

void bottom_panel_update(void)
    {
    char day[256], month[256], season[256], X, stati[256],
        mounted[256], minutes[256], result[256], temp[256], space_desc[256],
        room_desc[256], level_desc[256], mount_desc[256];
    long hours, monthday, year, elapsed;
    stati[0] = '\0';
    mounted[0] = '\0';
    minutes[0] = '\0';
    elapsed=game_variables->elapsed_units;
    /* The room string may sometimes be empty.  Here we check if it is empty, */
    /* and print the appropriate string.  */
    find_string(current_level->environment_name, level_desc);
    find_string(current_level->space[player->x][player->y].space_description,
      space_desc);
    find_string(current_level->space[player->x][player->y].room_description,
      room_desc);
    /* Capitalize first letter, if need be.  */
    if ((space_desc[0] >= 'a') && (space_desc[0] <= 'z'))
        space_desc[0] += 'A' - 'a';
    if (!(room_desc[0] == 0))
#ifdef HAS_SNPRINTF
        snprintf(result, 255, "%s in %s, in %s", space_desc, room_desc, level_desc);
#else
        sprintf(result, "%s in %s, in %s", space_desc, room_desc, level_desc);
#endif
    else
#ifdef HAS_SNPRINTF
        snprintf(result, 255, "%s, in %s", space_desc, level_desc);
#else
        sprintf(result, "%s, in %s", space_desc, level_desc);
#endif
    result[255] = '\0';
    temp[0] = '\0';
    if (flag_check(current_level->flags, PRINT_LEVEL_NUMBER))
        {
        if (current_level->power_level > 0)
#ifdef HAS_SNPRINTF
            snprintf(temp, 255, " (level %ld)", current_level->power_level);
#else
            sprintf(temp, " (level %ld)", current_level->power_level);
#endif
        else if (current_level->power_level < 0)
#ifdef HAS_SNPRINTF
            snprintf(temp, 255, " (floor %ld)", (0 - current_level->power_level));
#else
            sprintf(temp, " (floor %ld)", (0 - current_level->power_level));
#endif
        else
#ifdef HAS_SNPRINTF
            snprintf(temp, 255, " (ground floor)");
#else
            sprintf(temp, " (ground floor)");
#endif
        temp[255] = '\0';
        }
    strncat(result, temp, max(0, 255 - strlen(result)));
    result[255] = '\0';
    strncat(result, ".", max(0, 255 - strlen(result)));
    result[255] = '\0';
    mvwprintw(bottom_panel, 0, 0, result);
    wclrtoeol(bottom_panel);
    /* My modular arithmetic may be buggy */
#ifdef HAS_SNPRINTF
    snprintf(minutes, 255, "%ld%ld",
      ((((elapsed/(UNITS_PER_SECOND*60L)) % 60L)) / 10L),
      (((elapsed/(UNITS_PER_SECOND*60L)) % 60L) -
      (10L * (((elapsed/(UNITS_PER_SECOND*60L)) % 60L) / 10L))));
#else
    sprintf(minutes, "%ld%ld",
      ((((elapsed/(UNITS_PER_SECOND*60L)) % 60L)) / 10L),
      (((elapsed/(UNITS_PER_SECOND*60L)) % 60L) -
      (10L * (((elapsed/(UNITS_PER_SECOND*60L)) % 60L) / 10L))));
#endif
    minutes[255] = '\0';
    /* (60 + (((elapsed/(UNITS_PER_SECOND*60L)) % 60L)) / 10L),
    (60 + (((elapsed/(UNITS_PER_SECOND*60L)) % 60L)) -
    (10L * (10 + (((elapsed/(UNITS_PER_SECOND*60L)) % 60L) / 10L))))); */
    hours=((elapsed/(UNITS_PER_SECOND*3600L))%12);
    if (hours == 0)
        hours = 12;
    monthday=((game_variables->elapsed_days)%28);
    if (monthday == 0)
        monthday = 28;
    year=((game_variables->elapsed_days)/YEAR);
    if ((elapsed/(UNITS_PER_SECOND*3600L))%24 ==
    (elapsed/(UNITS_PER_SECOND*3600))%12) X = 'A'; else X = 'P';
    switch ((game_variables->elapsed_days)%7)
        {
        case 0: strncpy(day,"Sunday", 255); break;
        case 1: strncpy(day,"Monday", 255); break;
        case 2: strncpy(day,"Tuesday", 255); break;
        case 3: strncpy(day,"Wednesday", 255); break;
        case 4: strncpy(day,"Thursday", 255); break;
        case 5: strncpy(day,"Friday", 255); break;
        case 6: strncpy(day,"Saturday", 255); break;
        }
    day[255] = '\0';
    switch ((game_variables->elapsed_days/MONTH)%12)
        {
        case 0: strncpy(month,"January", 255); break;
        case 1: strncpy(month,"February", 255); break;
        case 2: strncpy(month,"March", 255); break;
        case 3: strncpy(month,"April", 255); break;
        case 4: strncpy(month,"May", 255); break;
        case 5: strncpy(month,"June", 255); break;
        case 6: strncpy(month,"July", 255); break;
        case 7: strncpy(month,"August", 255); break;
        case 8: strncpy(month,"September", 255); break;
        case 9: strncpy(month,"October", 255); break;
        case 10: strncpy(month,"November", 255); break;
        case 11: strncpy(month,"December", 255); break;
        }
    month[255] = '\0';
    switch (((game_variables->elapsed_days - MONTH)/84L)%4)
        {
        case 0: strncpy(season,"summer", 255); break;
        case 1: strncpy(season,"autumn", 255); break;
        case 2: strncpy(season,"winter", 255); break;
        case 3: strncpy(season,"spring", 255); break;
        }
    season[255] = '\0';
    mvwprintw(bottom_panel, 1, 0, "%ld:%s %cM, %s, %s %ld, %s, year %ld",
    hours, minutes, X, day, month, monthday, season, year);
    wclrtoeol(bottom_panel);
    /* Deal with stati here; I haven't put the descriptive code in yet. */
    /* Separate different stati by two spaces.  */
    stati[0] = '\0';
    if (player->extended->mount)
        {
    if (!(player->extended->mount->name == NULL))
        {
#ifdef HAS_SNPRINTF
        snprintf(mounted, 255, "riding %s  ",
          player->extended->mount->name->text);
#else
        sprintf(mounted, "riding %s  ",
          player->extended->mount->name->text);
#endif
        mounted[255] = '\0';
        }
    else
        {
        if (!(player->extended->mount->extended == NULL))
        strncpy(mount_desc, player->extended->mount->extended->species, 255);
        else
        strncpy(mount_desc,
          master_creatures[player->extended->mount->identity].species, 255);
        mount_desc[255] = '\0';
            if ((mount_desc[0] == 'a') ||
            (mount_desc[0] == 'e') ||
            (mount_desc[0] == 'i') ||
            (mount_desc[0] == 'o') ||
            (mount_desc[0] == 'u') ||
            (mount_desc[0] == 'A') ||
            (mount_desc[0] == 'E') ||
            (mount_desc[0] == 'I') ||
            (mount_desc[0] == 'O') ||
            (mount_desc[0] == 'U'))
#ifdef HAS_SNPRINTF
                snprintf(mounted, 255, "riding an %s  ", mount_desc);
#else
                sprintf(mounted, "riding an %s  ", mount_desc);
#endif
            else
#ifdef HAS_SNPRINTF
                snprintf(mounted, 255, "riding a %s  ", mount_desc);
#else
                sprintf(mounted, "riding a %s  ", mount_desc);
#endif
            mounted[255] = '\0';
            }
        }
    if (flag_check(player->flags, ABOVE_GROUND))
        {
        strncat(stati, "flying   ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_HUNGRY) > 0)
        switch(status_check(player, STATUS_HUNGRY))
            {
            case 1:
                strncat(stati, "hungry  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 2:
                strncat(stati, "very hungry  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 3:
                strncat(stati, "ravenous  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 4:
                strncat(stati, "weak  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            default:
                strncat(stati, "starving  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            }
    if (status_check(player, STATUS_THIRSTY) > 0)
        switch(status_check(player, STATUS_THIRSTY))
            {
            case 1:
                strncat(stati, "thirsty  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 2:
                strncat(stati, "very thirsty  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 3:
                strncat(stati, "dehydrated  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 4:
                strncat(stati, "severely dehydrated  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            default:
                strncat(stati, "dying of thirst  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            }
    if (flag_check(player->flags, SLEEPING))
        {
        strncat(stati, "asleep  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    else
        if (status_check(player, STATUS_TIRED) > 0)
            switch(status_check(player, STATUS_TIRED))
                {
                case 1:
                    strncat(stati, "drowsy  ", max(0, 255 - strlen(stati)));
                    stati[255] = '\0';
                break;
                case 2:
                    strncat(stati, "tired  ", max(0, 255 - strlen(stati)));
                    stati[255] = '\0';
                break;
                case 3:
                    strncat(stati, "very sleepy  ", max(0, 255 - strlen(stati)));
                    stati[255] = '\0';
                break;
                case 4:
                    strncat(stati, "barely awake  ", max(0, 255 - strlen(stati)));
                    stati[255] = '\0';
                break;
                default:
                    strncat(stati, "dangerously sleepy  ", max(0, 255 - strlen(stati)));
                    stati[255] = '\0';
                break;
                }
    if (status_check(player, STATUS_FATIGUED) > 0)
        switch(status_check(player, STATUS_FATIGUED))
            {
            case 1:
                strncat(stati, "fatigued  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 2:
                strncat(stati, "worn out  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 3:
                strncat(stati, "exhausted  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 4:
                strncat(stati, "totally exhausted  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            default:
                strncat(stati, "dangerously fatigued  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            }
    if (status_check(player, STATUS_COLD) > 0)
        switch(status_check(player, STATUS_COLD))
            {
            case 1:
                strncat(stati, "chilly  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 2:
                strncat(stati, "cold  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 3:
                strncat(stati, "chilled to the bone  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 4:
                strncat(stati, "hypothermic  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            default:
                strncat(stati, "dying of hypothermia  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            }
    if (status_check(player, STATUS_IMMOBILE) > 0)
        {
        strncat(stati, "immobile  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_LOST) > 0)
        {
        strncat(stati, "lost  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_PASSED_OUT) > 0)
        {
        strncat(stati, "passed out  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_STINKY) > 0)
        {
        strncat(stati, "stinky  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_ALCOHOL) > 0)
        switch(status_check(player, STATUS_ALCOHOL))
            {
            case 1:
                strncat(stati, "relaxed  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 2:
                strncat(stati, "merry  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 3:
                strncat(stati, "a bit too merry  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 4:
                strncat(stati, "rather tipsy  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
            break;
            case 5:
                strncat(stati, "drunk  ", max(0, 255 - strlen(stati)));
                stati[255] = '\0';
                flag_set(&player->flags, HAS_SINNED);
            break;
            }
    if (status_check(player, STATUS_ADRENALINE) > 0)
        {
        strncat(stati, "on adrenaline surge  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_BREATHE_WATER) > 0)
        {
        strncat(stati, "breathing water  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_COFFEE) > 0)
        {
        strncat(stati, "on caffeine  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_COFISA) > 0)
        {
        strncat(stati, "on cofisa tea  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_HELSRIANA) > 0)
        {
        strncat(stati, "on helsriana tea  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_HUMMER) > 0)
        {
        strncat(stati, "repelling animals  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    if (status_check(player, STATUS_TALINOR) > 0)
        {
        strncat(stati, "on talinor tea  ", max(0, 255 - strlen(stati)));
        stati[255] = '\0';
        }
    mvwprintw(bottom_panel, 2, 0, "%s%s", mounted, stati);
    wclrtoeol(bottom_panel);
#ifdef NO_WNOUTREFRESH
    wrefresh(bottom_panel);
#else
    wnoutrefresh(bottom_panel);
#endif
    }

long image_character(long x, long y)
    {
    long dispchar = ' ';
    long result, saw_creature;
    saw_creature = FALSE;
    if ((x < 0) || (x >= MAX_X) || (y < 0) || (y >= MAX_Y))
        return ' ';
    if (flag_check(current_level->space[x][y].flags, KNOWN))
        dispchar= current_level->space[x][y].image;
    if (can_see(player, x, y))
        {
        flag_set(&current_level->space[x][y].flags, KNOWN);
        if (current_level->space[x][y].creatures)
            {
            if (flag_check(current_level->space[x][y].creatures->current->
              flags, HIDDEN))
                {
                result = skill_check(player, player->extended->skills->
                  av[SEARCH], DIFFICULT,
                  0);
                if (result == 1)
                    {
                    dispchar=C_CANT_TELL;
                    saw_creature = TRUE;
                    }
                else if (result > 1)
                    {
                    dispchar = master_creatures[current_level->space[x][y].
                      creatures->current->identity].image;
                    saw_creature = TRUE;
                    flag_reset(&current_level->space[x][y].creatures->current->
                      flags, HIDDEN);
                    }
                }
            else
                {
                dispchar = master_creatures[current_level->space[x][y].
                  creatures->current->identity].image;
                saw_creature = TRUE;
                }
            }
        if (!(saw_creature))
            {
            if ((current_level->space[x][y].items)
              && (current_level != countryside))
                {
                if
                (current_level->space[x][y].items->next)
                    dispchar=C_PILE;
                else
                    dispchar=orig_items[current_level->space[x][y]
                      .items->current->identity].image;
                }
            else
                {
                if (((current_level->space[x][y].au) ||
                  (current_level->space[x][y].el) ||
                  (current_level->space[x][y].si) ||
                  (current_level->space[x][y].cu))
                  && (current_level != countryside))
                    dispchar=C_MONEY;
                else
                    {
                    if ((flag_check(current_level->space[x][y].
                      flags, S_CLOAKED_BY_MIST)) &&
                      (current_level->space[x][y].image==C_FLOOR))
                        dispchar=C_MIST;
                    else
                        dispchar=current_level->space[x][y].image;
                    }
                }
            }
        }
    else if (can_see_badly(player, x, y))
        {
        flag_set(&current_level->space[x][y].flags, KNOWN);
        if (current_level->space[x][y].creatures)
            {
            if (flag_check(current_level->space[x][y].creatures->current->
              flags, HIDDEN))
                {
                if (skill_check(player, player->extended->skills->av[SEARCH],
                  VERY_DIFFICULT,
                  0))
                    dispchar=C_CANT_TELL;
                }
            else
                dispchar=C_CANT_TELL;
            }
        if (dispchar != C_CANT_TELL)
            {
            if ((current_level->space[x][y].items)
              && (current_level != countryside))
                dispchar=C_CANT_TELL;
            else
                {
                if ((flag_check(current_level->space[x][y].flags,
                  S_CLOAKED_BY_MIST)) &&
                  (current_level->space[x][y].image==C_FLOOR))
                    dispchar=C_MIST;
                else
                    dispchar=current_level->space[x][y].image;
                }
            }
        }
    else if (current_level->space[x][y].creatures != NULL)
        {
        result = skill_check(player, player->extended->skills->av[HEAR_NOISES],
          distance(player->x, player->y, x, y) * current_level->scale +
          ONE_NOTCH * (!(flag_check(current_level->space[x][y].creatures->
          current->flags, INACTIVE))) + ONE_NOTCH *
          (!(clear_path(player->x, player->y, x, y))) + ONE_NOTCH *
          (flag_check(current_level->space[x][y].creatures->current->flags,
          HIDDEN)), 0);
        if (result == 1)
            {
            dispchar = 'X';
            saw_creature = TRUE;
            }
        else if (result > 1)
            {
            dispchar = master_creatures[current_level->space[x][y].creatures->
              current->identity].image;
            saw_creature = TRUE;
            }
        }

/*
 *
 * Player can always "see" himself, even if it's totally dark, he's in the
 * countryside, etc.
 *
 */

    if ((player->x == x) && (player->y == y))
        {
        dispchar=player->extended->image;
        flag_set(&current_level->space[x][y].flags, KNOWN);
        }
    if ((flag_check(current_level->space[x][y].flags, KNOWN)) &&
      (flag_check(game_variables->flags, ALWAYS_DISPLAY_STAIRCASES)) &&
      ((current_level->space[x][y].image == C_STAIRWELL_UP) ||
      (current_level->space[x][y].image == C_STAIRWELL_DOWN)))
        dispchar = current_level->space[x][y].image;
    if (saw_creature)
        {
        return dispchar | A_BOLD;
        }
    else
        {
        return dispchar | A_DIM;
        }
    }

void dungeon_display_update(void)
    {
    long dispchar;
    long i, j, k, l;
    if ((flag_check(game_variables->flags, PLAYER_AT_CENTER)) ||
      (abs(player->x - game_variables->display_center_x) > (COLS - 18) / 2 - 2)
      || (abs(player->y - game_variables->display_center_y) > (LINES - 12)/2 -
      2))
        {
        game_variables->display_center_x=player->x;
        game_variables->display_center_y=player->y;
        }
    for(i=0;i<COLS-16;++i)
        for(j=0;j<LINES-10;++j)
            {
            dispchar=C_UNKNOWN;
            if (!(((game_variables->display_center_x - (COLS-16)/2 + i) < 0) ||
            ((game_variables->display_center_x - (COLS-15)/2 + i) > MAX_X-1) ||
            ((game_variables->display_center_y - (LINES-10)/2 + j) < 0) ||
            ((game_variables->display_center_y - (LINES-9)/2 + j) > MAX_Y-1)))
                {
                k=i+game_variables->display_center_x-(COLS-16)/2;
                l=j+game_variables->display_center_y-(LINES-10)/2;
                dispchar = image_character(k, l);
                }
            mvwaddch(dungeon, j, i, dispchar);
            }
#ifdef NO_WNOUTREFRESH
    wrefresh(dungeon);
#else
    wnoutrefresh(dungeon);
#endif
    }

void top_text_update(void)
    {
#ifdef NO_WNOUTREFRESH
    wrefresh(top_text);
#else
    wnoutrefresh(top_text);
#endif
    }

void murefresh(void)
    {
    move(((player->y - game_variables->display_center_y) + 6 + (LINES-10)/2),
    ((player->x - game_variables->display_center_x) + 13 + (COLS-16)/2));
    /* Move the cursor to the player */
    if (flag_check(game_variables->flags, MAX_REFRESH))
        refresh();
    }

long morewait(void)
    {
    char c;
    mvwprintw(top_text, 5, 0, "-- More -- (hit space to continue)");
    wclrtoeol(top_text);
    wrefresh(top_text);
    for(c = tgetc();(!((c == ' ') || (c == 'q') || (c == ESCAPE)));putchar(7))
        {
        c = tgetc();
        fflush(stdout);
        }
    mvwprintw(top_text, 5, 0, "");
    wclrtoeol(top_text);
    wrefresh(top_text);
    return (!((c == 'q') || (c == 27)));
    }

long fs_morewait(void)
    {
    char c;
    mvprintw(LINES-1, 0, "-- More -- (hit space to continue)");
    clrtoeol();
    refresh();
    for(c=tgetc();(!((c == ' ') || (c == 'q') || (c == ESCAPE)));putchar(7))
        {
        c=tgetc();
        fflush(stdout);
        }
    mvprintw(LINES, 0, "");
    clrtoeol();
    refresh();
    return (!((c == 'q') || (c == 27)));
    }

long rf_fs_morewait(void)
    {
    char c;
    long my_continue, result;
    result = 0;
    mvprintw(LINES-1, 0, "-- More -- (hit space to continue, '?' for help)");
    clrtoeol();
    refresh();
    my_continue = TRUE;
    while (my_continue)
        {
        c = tgetc();
        my_continue = FALSE;
        switch(c)
            {
            case '?': case 'h':
                result = READ_FILE_HELP;
            break;
            case ' ': case 'f': case '+':
                result = READ_FILE_PAGE_DOWN;
            break;
            case '-': case 'p': case 'b':
                result = READ_FILE_PAGE_UP;
            break;
            case ESCAPE: case 'q':
                result = READ_FILE_QUIT;
            break;
            case REFRESH: case 12 /* ^L */:
                result = READ_FILE_REFRESH;
            break;
            case '\\':
                result = READ_FILE_SEARCH_BACKWARDS;
            break;
            case '/':
                result = READ_FILE_SEARCH_FORWARDS;
            break;
            default:
                putchar(7); /* Bell */
                my_continue = TRUE;
            break;
            }
        }
    mvprintw(LINES, 0, "");
    clrtoeol();
    refresh();
    return result;
    }

void fs_anykey(void)
    {
    mvprintw(LINES - 1, 0, "Hit any key to continue...");
    clrtoeol();
    refresh();
    tgetc();
    mvprintw(LINES - 1, 0, "");
    clrtoeol();
    refresh();
    }

void update_window(void)
    {
    left_panel_update();
    bottom_panel_update();
    top_text_update();
    dungeon_display_update();
    move(((player->y - game_variables->display_center_y) + 6 + (LINES-10)/2),
    ((player->x - game_variables->display_center_x) + 13 + (COLS-16)/2));
    refresh();
    }

long tmvgetstr(long y, long x, char *result) /* Having trouble with what */
                                                                                         /* curses provides.  */
    {
    long my_continue, position, success;
    char input;
    move(y, x);
    my_continue = TRUE;
    position = -1;
    success = 0;
    while (my_continue)
        {
        input = tgetc();
        switch(input)
            {
            case ESCAPE:
                result[0] = 0;
                my_continue = 0;
                success = 0;
            break;
            case 8: case 127:
                mvaddch(y, x + position, ' ');
                move(y, x + position);
                result[position] = 0;
                position = max(position - 1, -1);
            break;
            case 10: case 13:
                my_continue = 0;
                success = 1;
            break;
            default:
                ++position;
                mvaddch(y, x + position, input);
                result[position] = input;
                result[position + 1] = 0;
            break;
            }
        refresh();
        }
    move(y, x);
    return success;
    }

long tgets(char *result)
    {
    pmsg("");
    return tmvgetstr(4, 0, result);
    }

void projectile_path(long start_x, long start_y, long finish_x, long finish_y,
  char image)
    {
    long i, total, x, y, disp_x, disp_y;
    total = abs(start_x - finish_x) + abs(start_y - finish_y);
    for(i = 0; i < total; ++i)
        {
        x = start_x + finish_x * (i/total);
        y = start_y + finish_y * (i/total);
        if ((x != start_x) || (y != start_y))
            {
            disp_x=x+game_variables->display_center_x-(COLS-16)/2;
            disp_y=y+game_variables->display_center_y-(LINES-10)/2;
            if ((0 <= disp_x) && (disp_x < COLS - 16) && (0 <= disp_y) &&
              (disp_y < LINES - 10))
                {
                mvwaddch(dungeon, disp_y, disp_x, image);
                refresh();
                mvwaddch(dungeon, disp_y, disp_x, image_character(x, y));
                refresh();
                }
            }
        }
    }
