/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2003 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include <string.h>
#include "tms.h"

void p_northern_pool(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    pmsg("You wade in...");
    pmsg("The water is strangely warm.");
    pmsg("Drink from the water?");
    if (confirm_silent())
        {
        pmsg("The water is warm, and has a strong mineral flavor.");
        pmsg("You feel a strange tingle as you drink it.");
        morewait();
        flag_set(&game_variables->quests_completed, DRANK_LAKE);
        drink_water(c);
        status_add(c, STATUS_COLD, -100);
        }
    else
        pmsg("You decide not to risk it.");
    }

long p_puzzle_door(struct small_creature *c, long x, long y)
    {
    long my_continue, result;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    result = 0;
    if (current_level->space[x][y].property == P_CLOCKWORK_DOOR)
        {
        my_continue = TRUE;
        pmsg("You approach the clockwork door...");
        morewait();
        while (my_continue)
            {
            switch(current_level->space[x][y].other)
                {
                case PUZZLE_BOARD_COMPLETE_SQUARE: case
                  PUZZLE_BOARD_CONVERT_ADJACENT: case PUZZLE_BOARD_HIDDEN_MAZE:
                  case PUZZLE_BOARD_RANDOM: case PUZZLE_BOARD_SURROUND_OPPONENT:
                    result = board_game(current_level->space[x][y].other);
                break;
                case PUZZLE_PEG_ALL_BUT_LAST_DOWN: case PUZZLE_PEG_ALTERNATING:
                  case PUZZLE_PEG_CHECKSUM: case PUZZLE_PEG_RANDOM: case
                  PUZZLE_PEG_SYMMETRY:
                    result = peg_game(current_level->space[x][y].other);
                break;
                case PUZZLE_RIDDLE:
                    if (current_level->space[x][y].special != NULL)
                        {
                        if (current_level->space[x][y].special->text != NULL)
                            result = cryptogram_machine(
                              current_level->space[x][y].special->text);
                        else
                            pmsg("Error 1 in p_puzzle_door");
                        }
                    else
                        pmsg("Error 2 in p_puzzle_door");
                break;
                default:
                    pmsg("Error 3 in p_puzzle_door");
                    result = TRUE;
                break;
                }
            if (result)
                {
                my_continue = FALSE;
                gain_experience(c, UNITS_PER_EP / 10);
                pmsg("The door slides open.");
                flag_reset(&current_level->space[x][y].flags,
                  AUTOMATIC_EFFECT);
                flag_reset(&current_level->space[x][y].flags, DOOR_CLOSED);
                flag_set(&current_level->space[x][y].flags, CAN_OCCUPY);
                current_level->space[x][y].image = C_OPEN_DOOR;
                current_level->space[x][y].space_description = ST_AN_OPEN_DOOR;
                }
            else
                {
                pmsg("The door does not open.  Would you like to try again?");
                my_continue = confirm_silent();
                }
            }
        }
    return result;
    }

long p_donation(struct small_creature *c)
    {
    char choice, input[256];
    long amount, *coin, success;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    success = FALSE;
    coin = NULL;
    if (c != NULL)
        if (c->extended != NULL)
            {
            pmsg("Give (g)old, (e)lectrum, (s)ilver, or (c)opper?");
            choice = 'a';
            while ((choice != 'g') && (choice != 'e') && (choice != 's') &&
              (choice != 'c') && (choice != ESCAPE))
                choice = tgetc();
            if (choice != ESCAPE)
                {
                pmsg("How much?");
                tgets(input);
                sscanf(input, "%ld", &amount);
                switch(choice)
                    {
                    case 'g':
                        coin = &c->extended->au;
                    break;
                    case 'e':
                        coin = &c->extended->el;
                    break;
                    case 's':
                        coin = &c->extended->si;
                    break;
                    case 'c':
                        coin = &c->extended->cu;
                    break;
                    }
                if (amount <= 0)
                    pmsg("Aborted.");
                else if (*coin == 0)
                    pmsg("You don't have any of that kind of money.");
                else
                    {
                    if (amount > *coin)
                        pmsg("You don't have that much; giving what you have...");
                    prayer_units(c, 1);
                    *coin = max(0, *coin - amount);
                    success = TRUE;
                    }
                }
            else
                pmsg("Aborted.");
            }
    return success;
    }

void p_wrestle_blacksmith(void)
    {
    long success;
    struct small_item *dagger;
    if (!(flag_check(game_variables->flags, BLACKSMITH_GAVE_QUEST)))
        {
        pmsg("The blacksmith looks at you, and says, \"All right, ay'll give ye a quest.  Pin");
        nocap_pmsg("me or one of my colleagues in a wrestling match.\"");
        flag_set(&game_variables->flags, BLACKSMITH_GAVE_QUEST);
        }
    else
        pmsg("The blacksmith says to you, \"So, shall we wrestle?\"");
    pmsg("");
    pmsg("Wrestle him?");
    if (confirm_silent())
        {
        success = skill_check(player, player->extended->skills->av[WRESTLING],
          VERY_DIFFICULT, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        if (success < 0)
            {
            pmsg("Before you know what is going on, the blacksmith has you pinned.");
            pmsg("He pats you on the head, and says, \"Ye'll best be trainin' yerself, see?");
            pmsg("Ah'll still be here.\"");
            }
        else if (success == 0)
            {
            pmsg("After a long struggle, the blacksmith pins you.");
            pmsg("He nods to you, and says, \"Ye almost got me thar.  Ye just keep tryin'\"");
            }
        else if (success == 1)
            {
            pmsg("After a good match, you pin the blacksmith.  He bows before you.");
            flag_set(&game_variables->quests_completed, WRESTLED_BLACKSMITH);
            }
        else
            {
            pmsg("You pin the blacksmith with finesse.");
            pmsg("He gets up, and bows deeply.");
            morewait();
            pmsg("\"Ye be a mighty good wrestler thar.  'Tis not often I meet the likes of ye.");
            pmsg("And for that I give you this dagger.");
            pmsg("Ya come back here, ya hear me?\"");
            dagger = generate_item(current_level, DAGGER);
            small_item_make_normal(dagger);
            small_item_make_fine(dagger);
            add_item(&player->extended->pack, dagger);
            flag_set(&game_variables->quests_completed, WRESTLED_BLACKSMITH);
            }
        }
    morewait();
    }

void drink_water(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended != NULL)
        if (c->extended->stati->time[STATUS_THIRSTY] > (1 +
          adjustment_info[STATUS_THIRSTY].min) *
          adjustment_info[STATUS_THIRSTY].time_increment)
            {
            status_add(c, STATUS_THIRSTY, -100);
            if (c == player)
                pmsg("You drink your fill.");
            }
    }

void p_restaurant(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (current_level->space[c->x][c->y].special != NULL)
        if (current_level->space[c->x][c->y].special->establishment != NULL)
            p_establishment(c, RESTAURANT,
              current_level->space[c->x][c->y].special->establishment);
    }

void p_tavern(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (current_level->space[c->x][c->y].special != NULL)
        if (current_level->space[c->x][c->y].special->establishment != NULL)
            p_establishment(c, TAVERN,
              current_level->space[c->x][c->y].special->establishment);
    }

void p_tavern_seat(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if ((c == player) && (current_level->establishment != NULL))
        p_establishment(c, TAVERN, current_level->establishment);
    }

void p_establishment(struct small_creature *c, long establishment_id,
  struct establishment_struct *establishment)
    {
    long i, j, my_continue, print_message, desired_item, item_flags;
    char message[256], temp[256], player_input[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    my_continue = TRUE;
    while (my_continue)
        {
        clear();
        switch(establishment_id)
            {
            case RESTAURANT:
                fs_pmsg("You are inside a restaurant.  Would you like to: ");
            break;
            case TAVERN:
                fs_pmsg("You are inside a tavern.  Would you like to: ");
            break;
            }
        fs_pmsg("");
        fs_pmsg("a: list foods and drinks available");
        fs_pmsg("b: make a purchase");
        fs_pmsg("c: neither");
        switch(tgetc())
            {
            case 'a':
                clear();
                print_message = TRUE;
                for(i = j = 0; i < NUM_ESTABLISHMENT_ITEMS; ++i)
                    {
                    if (flag_check(establishment->flags[i],
                      ESTABLISHMENT_ITEM_AVAILABLE))
                        {
                        ++j;
#ifdef HAS_SNPRINTF
                        snprintf(message, 255, "%ld: ", j);
#else
                        sprintf(message, "%ld: ", j);
#endif
                        message[255] = '\0';
                        find_string(establishment_info[i].string, temp);
                        strncat(message, temp, max(0, 255 - strlen(message)));
                        strncat(message, " (", max(0, 255 - strlen(message)));
                        money_string(establishment->price[i], temp);
                        strncat(message, temp, max(0, 255 - strlen(message)));
                        strncat(message, ")", max(0, 255 - strlen(message)));
                        if (print_message)
                            print_message &= fs_pmsg(message);
                        }
                    }
                fs_anykey();
            break;
            case 'b':
                clear();
                fs_pmsg("");
                fs_pmsg("Which item?");
                tgets(player_input);
                sscanf(player_input, "%ld", &desired_item);
                for(i = j = 0; i < NUM_ESTABLISHMENT_ITEMS; ++i)
                    {
                    if (flag_check(establishment->flags[i],
                      ESTABLISHMENT_ITEM_AVAILABLE))
                        {
                        ++j;
                        if (j == desired_item)
                            {
                            if (deduct_amount(c, establishment->price[i]))
                                {
                                item_flags = establishment_info[i].flags;
                                if (flag_check(item_flags, ESTABLISHMENT_FOOD))
                                    pmsg("Yum!");
                                else
                                    pmsg("You sip it...");
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_ALCOHOL))
                                    status_add(c, STATUS_ALCOHOL, 1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_CAFFEINE))
                                    status_add(c, STATUS_COFFEE, 1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_COFISA_TEA))
                                    status_add(c, STATUS_COFISA, 1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_HELSRIANA_TEA))
                                status_add(c, STATUS_HELSRIANA, 1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_TALINOR_TEA))
                                    status_add(c, STATUS_TALINOR, 1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_FIREBREATHER))
                                    {
                                    fs_pmsg("Your mouth feels like it is on fire!");
                                    status_add(c, STATUS_THIRSTY, 1);
                                    }
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_QUENCH_THIRST))
                                    status_add(c, STATUS_THIRSTY, -1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_MAKE_SLEEPY))
                                    status_add(c, STATUS_TIRED, 1);
                                if (flag_check(item_flags,
                                  ESTABLISHMENT_WARM_UP))
                                    status_add(c, STATUS_COLD, -1);
                                if (flag_check(item_flags, ESTABLISHMENT_FOOD))
                                    status_add(c, STATUS_HUNGRY, -1);
                                }
                            else
                                pmsg("You don't have enough money for that.");
                            }
                        }
                    }
                fs_anykey();
            break;
            case 'c': case 'q': case 'Q': case ESCAPE:
                my_continue = FALSE;
                clear();
            break;
            }
        }
    }

void p_collection_box(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        {
        pmsg("You are at a collection box.  Would you like to make a donation?");
        if (confirm_silent())
            p_donation(c);
        }
    }

void p_prayer_table(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        {
        pmsg("You are at a table where you may light a candle to symbolize your ascending");
        nocap_pmsg("prayers.  Would you like to light a candle?  (The suggested donation is one");
        nocap_pmsg("silver crown.");
        if (confirm_silent())
            {
            pmsg("You light a candle as you pray.");
            pmsg("Your prayers ascend with the flame...");
            prayer_units(c, 1);
            p_donation(c);
            }
        }
    }

void p_house(void)
    {
    char child_name[256];
    if ((current_level->race == JEC) && (game_variables->current_quest ==
      NAMED_CHILD) && (!(flag_check(game_variables->quests_completed,
      NAMED_CHILD))))
        {
        pmsg("A child has just been born, and you tell of your quest.");
        pmsg("What would you like to name the child?");
        if (tgets(child_name))
            flag_set(&game_variables->quests_completed, NAMED_CHILD);
        }
    else
        pmsg("This is just the door to somebody's house.");
    }

void p_hedge(struct small_creature *c)
    {
    long success;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    success = skill_check(c, c->def, MODERATE,
      SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
    if (success >= 1)
        {
        if (c == player)
            pmsg("You slide through the hedge...");
        }
    else
        {
        if (c == player)
            pmsg("Ouch.  That scratches!");
        if (success < 0)
            {
            if (c == player)
                pmsg("You're thtuck!");
            if (c->extended != NULL)
                status_add(c, STATUS_IMMOBILE, 1);
            else
                flag_set(&c->flags, IMMOBILE);
            }
        if (creature_damage(c, -30, CREATURE_DAMAGE_WAKE_UP, 3))
            creature_death(c);
        }
    }

void p_bed(struct small_creature *c)
    {
    struct clist *traverse;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        {
        pmsg("You are on a bed; would you like to go to sleep?");
        if (confirm_silent())
            {
            if (current_level->establishment != NULL) /* We are in an inn */
                for(traverse = current_level->creatures; traverse != NULL;
                  traverse = traverse->next)
        {
        if (traverse->current != NULL)
                    go_to_sleep(traverse->current);
        }
            else
                go_to_sleep(c);
            if (c->extended != NULL)
                pass_time(c->extended->stati->time[STATUS_SLEEPING]);
            }
        }
    }

void p_rubble(struct small_creature *c)
    {
    long result;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if ((c->extended != NULL) && (flag_check(c->flags, ABOVE_GROUND)))
        {
        result = skill_check(c, c->extended->skills->av[CLIMBING], MODERATE,
          SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
        if (result >= 2)
            {
            if (c == player)
                pmsg("You climb along with confidence!");
            }
        else if (result == 1)
            {
            if (c == player)
                pmsg("You climb along...");
            }
        else if (result == 0)
            {
            if (c == player)
                pmsg("As you climb the rubble, it slides, and you are trapped!");
            status_add(c, STATUS_IMMOBILE, 1);
            }
        else
            {
            if (c == player)
                {
                pmsg("As you climb the rubble, it slides, and you are trapped!");
                pmsg("You fall and injure yourself!");
                }
            status_add(c, STATUS_IMMOBILE, 1 - result);
            if (creature_damage(c, 0 - 10 - (10 * result),
              CREATURE_DAMAGE_WAKE_UP, 0 - result))
                creature_death(c);
            }
        }
    }

long p_inn_entrance(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        {
        pmsg("This is the entrance to the rooms; the charge is one electrum going in for the");
        nocap_pmsg("night, and one electrum leaving in the morning.  Go in/out?");
        if (confirm_silent())
            {
            deduct_amount(c, EL);
            return 1;
            }
        else
            {
            pmsg("You back away.");
            return 0;
            }
        }
    else
        return 1;
    }

void archery_tournament(struct small_creature *c)
    {
    char message[256];
    long contestant[50], still_in[50], score[50], i, j, num_contestants,
      my_continue, min_score, max_score, which_round;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c->extended != NULL)
        {
        contestant[0] = max(c->extended->skills->av[ATTACK_LONGBOW],
          c->extended->skills->av[ATTACK_CROSSBOW]);
        for(i = 1; i < 50; ++i)
            {
            contestant[i] = 0;
            while (trandom() % 2 != 0)
                contestant[i] += ONE_QUARTER_NOTCH;
            }
        num_contestants = 10;
        while (trandom() % 10 != 0)
            ++num_contestants;
        num_contestants = min(num_contestants, 50);
        for(i = 0; i < 50; ++i)
            if (i < num_contestants)
                still_in[i] = TRUE;
            else
                still_in[i] = FALSE;
        clear();
        read_file(FILE_ARCHERY_TOURNAMENT_RULES);
        clear();
        fs_pmsg("You are contestant one.");
        fs_anykey();
        my_continue = TRUE;
        which_round = 1;
        while (my_continue)
            {
            clear();
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "Round %ld:", which_round);
#else
            sprintf(message, "Round %ld:", which_round);
#endif
            message[255] = '\0';
            fs_pmsg(message);
            fs_pmsg("");
            min_score = 100;
            max_score = 0;
            for(i = 0; i < 50; ++i)
                {
                if (still_in[i])
                    {
                    score[i] = 0;
                    for(j = 0; j < 5; ++j)
                        score[i] += max(0, skill_check(c, contestant[i],
                          MODERATE,
                          (SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE * (i ==
                          0))));
                    min_score = min(score[i], min_score);
                    max_score = max(score[i], max_score);
#ifdef HAS_SNPRINTF
                    snprintf(message, 255, "Contestant %ld: score %ld", i + 1,
                      score[i]);
#else
                    sprintf(message, "Contestant %ld: score %ld", i + 1,
                      score[i]);
#endif
                    message[255] = '\0';
                    fs_pmsg(message);
                    }
                }
            fs_pmsg("");
            if (min_score != max_score)
                {
                fs_pmsg("Removed:");
                for(i = 0; i < 50; ++i)
                    if (still_in[i])
                        if (score[i] == min_score)
                            {
                            still_in[i] = FALSE;
#ifdef HAS_SNPRINTF
                            snprintf(message, 255, "Contestant %ld", i + 1);
#else
                            sprintf(message, "Contestant %ld", i + 1);
#endif
                            message[255] = '\0';
                            if (i == 0)
                                strncat(message, " (you)", max(0, 255 - strlen(message)));
                            message[255] = '\0';
                            strncat(message, ".", max(0, 255 - strlen(message)));
                            message[255] = '\0';
                            fs_pmsg(message);
                            }
                }
            else
                {
                num_contestants = 0;
                for(i = 0; i < 50; ++i)
                    if (still_in[i])
                        ++num_contestants;
                if (num_contestants > 1)
                    fs_pmsg("The round is a draw; no contestants removed.");
                else
                    {
                    for(i = 0; i < 50; ++i)
                        if (still_in[i])
                            {
#ifdef HAS_SNPRINTF
                            snprintf(message, 255, "The champion: %ld!", i + 1);
#else
                            sprintf(message, "The champion: %ld!", i + 1);
#endif
                            message[255] = '\0';
                            fs_pmsg(message);
                            if (i == 0)
                                {
                                fs_pmsg("You won!");
                                flag_set(&game_variables->quests_completed,
                                  WON_ARCHERY);
                                }
                            else
                                fs_pmsg("(You did not win this time.)");
                            }
                    my_continue = 0;
                    }
                }
            fs_anykey();
            }
        }
    }

void p_fire(struct small_creature *c)
    {
    char message[256], creature_name[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (can_see(player, c->x, c->y))
        {
        if (c == player)
            pmsg("You are burned by fire!");
        else
            {
            find_title(c, creature_name);
#ifdef HAS_SNPRINTF
            snprintf(message, 255, "%s is burned by fire!", creature_name);
#else
            sprintf(message, "%s is burned by fire!", creature_name);
#endif
            message[255] = '\0';
            pmsg(message);
            }
        }
    if (creature_damage(c, 0, CREATURE_DAMAGE_WAKE_UP, 2))
        creature_death(c);
    }

void p_inscription(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        {
        pmsg("There is an inscription on the door before you.");
        pmsg("The inscription reads:");
        pmsg("");
        pmsg("\"Welcome, hero.");
        pmsg("A hidden danger now approaches.");
    pmsg("Only the ultimate sacrifice can save the land.\"");
        }
    }

void p_final_test(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        {
        read_file(FILE_FINAL_QUEST_TEST);
        clear();
        fs_pmsg("Take the angel's hand?");
        if (!(confirm_silent()))
            {
            if (flag_check(c->flags, HAS_SINNED))
                {
                clear();
                fs_pmsg("You are utterly consumed.");
                player_death();
                }
            else
                {
                read_file(FILE_FINAL_QUEST_WIN);
                current_level->space[c->x][c->y].special = new_special();
                current_level->space[c->x][c->y].special->takes_player_to =
                  main_city;
                current_level->space[c->x][c->y].special->x = 0;
                current_level->space[c->x][c->y].special->y = 0;
                change_level(c->x, c->y);
                flag_set(&game_variables->quests_completed, FINAL_QUEST);
                }
            }
        else
            {
            read_file(FILE_FINAL_QUEST_LOSE);
            player_death();
            }
        }
    else
        creature_death(c);
    }

void p_mapmaker(struct small_creature *c)
    {
    long amount, i, j, k, city_x, city_y;
    char player_input[256];
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        if (c->extended != NULL)
            {
            pmsg("You are at a mapmaker's shop.  The minimum price for a map is 200 gold.");
            pmsg("Would you like to buy a map?");
            if (confirm_silent())
                {
                pmsg("How much (in increments of 50) do you wish to pay for the map?");
                if (tgets(player_input))
                    {
                    sscanf(player_input, "%ld", &amount);
                    if (amount < 200)
                        pmsg("Aborted.");
                    else if (deduct_amount(c, amount * AU))
                        {
                        if (current_level->space[0][0].special != NULL)
                            {
                            city_x = current_level->space[0][0].special->x;
                            city_y = current_level->space[0][0].special->y;
                            }
                        else
                            {
                            city_x = MAX_X/2;
                            city_y = MAX_Y/2;
                            }
                        for(i = 0; i < MAX_X; ++i)
                            for(j = 0; j < MAX_Y; ++j)
                                {
                                if (((city_x - i) * (city_x - i)) + ((city_y -
                                  j) * (city_y - j)) <= (((amount - 100) / 50) *
                                  ((amount - 100) / 50)))
                                    {
                                    if (flag_check(countryside->space[i][j].flags, HAS_SECRET))
                                        {
                                        change_space(&countryside->space[i][j],
                                          countryside->space[i][j].other);
                                        }
                                    flag_set(&countryside->space[i][j].flags,
                                      KNOWN);
                                    }
                                if (flag_check(current_level->space[i][j].flags, HAS_SECRET))
                                    {
                                    change_space(&current_level->space[i][j], current_level->space[i][j].other);
                                    }
                                flag_set(&current_level->space[i][j].flags,
                                  KNOWN);
                                for(k = 0; k < NUM_LOCATION_TYPES; ++k)
                                    if (flag_check(current_level->flags, IS_CITY))
                                        if (flag_check(
                                          current_level->space[i][j].flags,
                                          OTHER_IS_LOCATION_TYPE) &&
                                          (current_level->space[i][j].other
                                          == current_level->location[k].identity))
                                            if (!(flag_check(current_level->
                                              location[k].flags,
                                              LOCATION_KNOWN)))
                                                {
                                                flag_set(
                                                  &current_level->location[k].
                                                  flags, LOCATION_KNOWN);
                                                current_level->location[k].x = i;
                                                current_level->location[k].y = j;
                                                }
                                }
                        pmsg("You now have the city and the surrounding area mapped out.");
                        }
                    else
                        pmsg("You don't have that much money!");
                    }
                }
            }
    }

void p_dry_tree(struct small_creature *c, long x, long y)
    {
    long result;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c != NULL)
        if (c->extended != NULL)
            {
            result = skill_check(c, c->extended->skills->av[CLIMBING],
              MODERATE, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
            if (result >= 1)
                {
                if (c == player)
                    pmsg("You find some firewood!");
                acquire_item(c, FIREWOOD);
                current_level->space[x][y].property = P_GREEN_TREE;
                }
            else if (result == 0)
                {
                if (c == player)
                    pmsg("You can't seem to get up the tree.");
                }
            else
                {
                if (c == player)
                    pmsg("When you try to climb, you slip and fall!");
                if (creature_damage(c, (0 - 10 - 10 * result),
                  CREATURE_DAMAGE_WAKE_UP, 0 - result))
                    creature_death(c);
                }
            }
    }

void p_fruit_tree(struct small_creature *c, long x, long y)
    {
    long result;
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c != NULL)
        if (c->extended != NULL)
            {
            result = skill_check(c, c->extended->skills->av[CLIMBING],
              MODERATE, SKILL_CHECK_CRITICAL_SUCCESS_GAIN_EXPERIENCE);
            if (result >= 1)
                {
                if (c == player)
                    pmsg("You find some fruit!");
                acquire_item(c, TREE_FRUIT);
                current_level->space[x][y].property = P_GREEN_TREE;
                }
            else if (result == 0)
                {
                if (c == player)
                    pmsg("You can't seem to get up the tree.");
                }
            else
                {
                if (c == player)
                    pmsg("When you try to climb, you slip and fall!");
                if (creature_damage(c, (0 - 10 - 10 * result),
                  CREATURE_DAMAGE_WAKE_UP, 0 - result))
                    creature_death(c);
                }
            }
    }

void p_green_tree(struct small_creature *c, long x, long y)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if (c == player)
        pmsg("This tree seems to have neither fruit nor firewood to gather.");
    }

void p_tinkerers_guild(struct small_creature *c)
    {
#ifdef DEBUG
    check_pointer((struct pointer_struct *) c);
#endif
    if ((c == player) || (is_following(c, player)))
        {
        pmsg("The Tinkerer's Guild.");
        pmsg("");
        if (!(flag_check(game_variables->flags,
          PLAYER_JOINED_TINKERERS_GUILD)))
            {
            pmsg("Do you wish to join (1000 gold)?");
            if (confirm_silent())
                {
                if (deduct_amount(c, 1000L * AU ))
                    {
                    flag_set(&game_variables->flags,
                      PLAYER_JOINED_TINKERERS_GUILD);
                    pmsg("Would you like to make something?");
                    if (confirm_silent())
                        make_item(c, TINKERING);
                    }
                else
                    pmsg("Looks like you don't yet have the cash.");
                }
            }
        else
            {
            pmsg("Would you like to make something?");
            if (confirm_silent())
                make_item(c, TINKERING);
            }
        }
    }
