/***************************************************************************/
/* The Minstrel's Song is (c) 1998-2003 by Jonathan Hayward.  See the file */
/* lib/copyrigh.txt for a copyright and license statement.                 */
/***************************************************************************/

#include "tms.h"
#include <math.h>

void find_pronoun(struct small_creature *c, char *result)
    {
    if (c != NULL)
        {
        if (flag_check(c->flags, HUMAN))
            {
            if (c->extended != NULL)
                switch(c->extended->gender)
                    {
                    case MALE:
                        strncpy(result, "he", 255);
                    break;
                    case FEMALE:
                        strncpy(result, "she", 255);
                    break;
                    default:
                        strncpy(result, "it", 255);
                    break;
                    }
            else
                switch(trandom() % 2)
                    {
                    case 0:
                        strncpy(result, "he", 255);
                    break;
                    default:
                        strncpy(result, "she", 255);
                    break;
                    }
            }
        else
            strncpy(result, "it", 255);
        }
    else
        strncpy(result, "", 255);
    result[255] = '\0';
    }

void check_creature(struct small_creature *c)
    {
#ifdef DEBUG
    char message[256];
#endif
    c = c; /* For if compiled without DEBUG defined.  */
#ifdef DEBUG
    if ((c->identity < 0) || (c->identity >= NUM_CREATURES))
        {
#ifdef HAS_SNPRINTF
        snprintf(message, 255, "Out of range creature identity %ld!", c->identity);
#else
        sprintf(message, "Out of range creature identity %ld!", c->identity);
#endif
        message[255] = '\0';
        string_check_exit(message);
        }
    else if ((c->x < 0) || (c->x >= MAX_X) || (c->y < 0) || (c->y >= MAX_Y))
        {
#ifdef HAS_SNPRINTF
        snprintf(message, 255, "Out of range creature x or y (%ld, %ld)!", c->x,
          c->y);
#else
        sprintf(message, "Out of range creature x or y (%ld, %ld)!", c->x,
          c->y);
#endif
        message[255] = '\0';
        string_check_exit(message);
        }
#endif
    }

void string_check_exit(char *message)
    {
#ifdef DEBUG
    long x;
#endif
    clear();
    fs_pmsg(message);
#ifdef DEBUG
    fs_pmsg("Exit and dump core?");
    if (confirm_silent())
        {
        fs_pmsg("Dumping core to enable analysis...");
        x = 1 / 0;
        }
#endif
    }

void check_xy(long x, long y)
    {
#ifdef DEBUG
    char message[256];
#endif
#ifdef DEBUG
    if ((x < 0) || (x >= MAX_X))
        {
#ifdef HAS_SNPRINTF
        snprintf(message, 255, "Out of range x value: %ld!", x);
#else
        sprintf(message, "Out of range x value: %ld!", x);
#endif
        message[255] = '\0';
        string_check_exit(message);
        }
    if ((y < 0) || (y >= MAX_Y))
        {
#ifdef HAS_SNPRINTF
        snprintf(message, 255, "Out of range y value: %ld!", y);
#else
        sprintf(message, "Out of range y value: %ld!", y);
#endif
        message[255] = '\0';
        string_check_exit(message);
        }
#endif
    }

void check_string(char *string)
    {
    string = string;
#ifdef DEBUG
    if (strlen(string) > 254)
        {
        printw("\nExcessive string length %ld: string:\n%s\n", strlen(string),
          string);
        refresh();
        }
#endif
    }

void check_pointer(struct pointer_struct *test_pointer)
    {
#ifdef DEBUG
    long foo;
#endif
    if ((test_pointer != NULL) && (!(flag_check(game_variables->flags,
      DURING_INIT))))
        {
#ifdef DEBUG
        /*
        if ((abs((int) test_pointer) < 0xffff) || (abs(((int) test_pointer) -
          0xffffff) < 0xff))
            {
            pmsg("Error! Invalid pointer.  Dump core?");
            if (confirm_silent())
                foo = 1 / 0;
            }
        else */ if (test_pointer != test_pointer->me)
            {
            pmsg("Error! Invalid pointer.  Dump core?");
            if (confirm_silent())
                foo = 1 / 0;
            }
#endif
        }
    }

void check_level_pointers(struct level *l)
    {
    long i, j;
    struct clist *cl;
    struct ilist *il;
    for(i = 0; i < MAX_X; ++i)
        for(j = 0; j < MAX_Y; ++j)
            {
            if (l->space[i][j].creatures != NULL)
                for(cl = l->space[i][j].creatures; cl != NULL; cl = cl->next)
                    {
                    check_pointer((struct pointer_struct *) cl);
                    check_pointer((struct pointer_struct *) cl->current);
                    check_creature(cl->current);
                    }
            if (l->space[i][j].items != NULL)
                for(il = l->space[i][j].items; il != NULL; il = il->next)
                    {
                    check_pointer((struct pointer_struct *) il);
                    check_pointer((struct pointer_struct *) il->current);
                    check_pointer((struct pointer_struct *)
                      il->current->writing);
                    check_pointer((struct pointer_struct *) il->current->tname);
                    }
            if (l->space[i][j].special != NULL)
                if (l->space[i][j].special->shop != NULL)
                    {
                    if (l->space[i][j].special->shop->creatures != NULL)
                        for(cl = l->space[i][j].special->shop->creatures; cl
                          != NULL; cl = cl->next)
                            {
                            check_pointer((struct pointer_struct *) cl);
                            check_pointer((struct pointer_struct *)
                              cl->current);
                            check_creature(cl->current);
                            }
                    if (l->space[i][j].special->shop->items != NULL)
                        for(il = l->space[i][j].special->shop->items; il !=
                          NULL; il = il->next)
                            {
                            check_pointer((struct pointer_struct *) il);
                            check_pointer((struct pointer_struct *)
                              il->current);
                            check_pointer((struct pointer_struct *)
                              il->current->writing);
                            check_pointer((struct pointer_struct *)
                              il->current->tname);
                            }
                    }
            }
    }

void my_finish(long status)
    {
    if (status == 0)
        {
        clear();
        printw("\n\nBye!\n");
        fs_morewait();
        }
    nocbreak();
    echo();
    refresh();
    endwin();
    exit(status);
    }
